@@ -46,12 +46,11 @@ t_canvas *canvas_list;              /* list of all root canvases */
 static void canvas_start_dsp(void);
 static void canvas_stop_dsp(void);
 static void canvas_drawlines(t_canvas *x);
-static void canvas_setbounds(t_canvas *x, int x1, int y1, int x2, int y2);
-static void canvas_reflecttitle(t_canvas *x);
+static void canvas_dosetbounds(t_canvas *x, int x1, int y1, int x2, int y2);
+void canvas_reflecttitle(t_canvas *x);
 static void canvas_addtolist(t_canvas *x);
 static void canvas_takeofflist(t_canvas *x);
 static void canvas_pop(t_canvas *x, t_floatarg fvis);
-void canvas_create_editor(t_glist *x, int createit);
 
 /* --------- functions to handle the canvas environment ----------- */
 
@@ -60,44 +59,11 @@ static t_symbol *canvas_newdirectory = &s_;
 static int canvas_newargc;
 static t_atom *canvas_newargv;
 
-static void glist_doupdatewindowlist(t_glist *gl, char *sbuf)
-{
-    t_gobj *g;
-    if (!gl->gl_owner)
-    {
-        /* this is a canvas; if we have a window, put on "windows" list */
-        t_canvas *canvas = (t_canvas *)gl;
-        if (canvas->gl_havewindow)
-        {
-            if (strlen(sbuf) + strlen(gl->gl_name->s_name) + 100 <= 1024)
-            {
-                char tbuf[1024];
-                sprintf(tbuf, "{{%s} .x%lx} ", gl->gl_name->s_name,
-                    (t_int)canvas);
-                strcat(sbuf, tbuf);
-            }
-        }
-    }
-    for (g = gl->gl_list; g; g = g->g_next)
-    {
-        if (pd_class(&g->g_pd) == canvas_class)
-            glist_doupdatewindowlist((t_glist *)g, sbuf);
-    }
-    return;
-}
-
     /* maintain the list of visible toplevels for the GUI's "windows" menu */
 void canvas_updatewindowlist( void)
 {
-    t_canvas *x;
-    char sbuf[1024];
-    strcpy(sbuf, "set menu_windowlist {");
-        /* find all root canvases */
-    for (x = canvas_list; x; x = x->gl_next)
-        glist_doupdatewindowlist(x, sbuf);
-    /* next line updates the window menu state before -postcommand tries it */
-    strcat(sbuf, "}\npdtk_fixwindowmenu\n");
-    sys_gui(sbuf);
+    if (! glist_amreloadingabstractions)  /* not if we're in a reload */         
+        sys_gui("::pd_menus::update_window_menu\n");
 }
 
     /* add a glist the list of "root" canvases (toplevels without parents.) */
@@ -158,7 +124,7 @@ t_canvasenvironment *canvas_getenv(t_canvas *x)
     if (!x) bug("canvas_getenv");
     while (!x->gl_env)
         if (!(x = x->gl_owner))
-            bug("t_canvasenvironment", x);
+            bug("t_canvasenvironment");
     return (x->gl_env);
 }
 
@@ -233,7 +199,7 @@ void canvas_rename(t_canvas *x, t_symbol *s, t_symbol *dir)
     x->gl_name = s;
     if (strcmp(x->gl_name->s_name, "Pd"))
         pd_bind(&x->gl_pd, canvas_makebindsym(x->gl_name));
-    if (glist_isvisible(x))
+    if (x->gl_havewindow)
         canvas_reflecttitle(x);
     if (dir && dir != &s_)
     {
@@ -398,7 +364,7 @@ t_canvas *canvas_new(void *dummy, t_symbol *sel, int argc, t_atom *argv)
     x->gl_y1 = 0;
     x->gl_x2 = 1;
     x->gl_y2 = 1;
-    canvas_setbounds(x, xloc, yloc, xloc + width, yloc + height);
+    canvas_dosetbounds(x, xloc, yloc, xloc + width, yloc + height);
     x->gl_owner = owner;
     x->gl_name = (*s->s_name ? s : 
         (canvas_newfilename ? canvas_newfilename : gensym("Pd")));
@@ -448,8 +414,8 @@ static void canvas_coords(t_glist *x, t_symbol *s, int argc, t_atom *argv)
     /* make a new glist and add it to this glist.  It will appear as
     a "graph", not a text object.  */
 t_glist *glist_addglist(t_glist *g, t_symbol *sym,
-    float x1, float y1, float x2, float y2,
-    float px1, float py1, float px2, float py2)
+    t_float x1, t_float y1, t_float x2, t_float y2,
+    t_float px1, t_float py1, t_float px2, t_float py2)
 {
     static int gcount = 0;
     int zz;
@@ -474,7 +440,7 @@ t_glist *glist_addglist(t_glist *g, t_symbol *sym,
         that is higher on the screen. */
     if (py2 < py1)
     {
-        float zz;
+        t_float zz;
         zz = y2;
         y2 = y1;
         y1 = zz;
@@ -511,8 +477,6 @@ t_glist *glist_addglist(t_glist *g, t_symbol *sym,
     if (!menu)
         pd_pushsym(&x->gl_pd);
     glist_add(g, &x->gl_gobj);
-    if (glist_isvisible(g))
-        canvas_create_editor(x, 1);
     return (x);
 }
 
@@ -520,14 +484,14 @@ t_glist *glist_addglist(t_glist *g, t_symbol *sym,
 void glist_glist(t_glist *g, t_symbol *s, int argc, t_atom *argv)
 {
     t_symbol *sym = atom_getsymbolarg(0, argc, argv);   
-    float x1 = atom_getfloatarg(1, argc, argv);  
-    float y1 = atom_getfloatarg(2, argc, argv);  
-    float x2 = atom_getfloatarg(3, argc, argv);  
-    float y2 = atom_getfloatarg(4, argc, argv);  
-    float px1 = atom_getfloatarg(5, argc, argv);  
-    float py1 = atom_getfloatarg(6, argc, argv);  
-    float px2 = atom_getfloatarg(7, argc, argv);  
-    float py2 = atom_getfloatarg(8, argc, argv);
+    t_float x1 = atom_getfloatarg(1, argc, argv);  
+    t_float y1 = atom_getfloatarg(2, argc, argv);  
+    t_float x2 = atom_getfloatarg(3, argc, argv);  
+    t_float y2 = atom_getfloatarg(4, argc, argv);  
+    t_float px1 = atom_getfloatarg(5, argc, argv);  
+    t_float py1 = atom_getfloatarg(6, argc, argv);  
+    t_float px2 = atom_getfloatarg(7, argc, argv);  
+    t_float py2 = atom_getfloatarg(8, argc, argv);
     glist_addglist(g, sym, x1, y1, x2, y2, px1, py1, px2, py2);
 }
 
@@ -540,7 +504,14 @@ int glist_isgraph(t_glist *x)
 
     /* This is sent from the GUI to inform a toplevel that its window has been
     moved or resized. */
-static void canvas_setbounds(t_canvas *x, int x1, int y1, int x2, int y2)
+static void canvas_setbounds(t_canvas *x, t_float left, t_float top, 
+                             t_float right, t_float bottom)
+{
+    canvas_dosetbounds(x, (int)left, (int)top, (int)right, (int)bottom);
+}
+
+/* this is the internal version using ints */
+static void canvas_dosetbounds(t_canvas *x, int x1, int y1, int x2, int y2)
 {
     int heightwas = y2 - y1;
     int heightchange = y2 - y1 - (x->gl_screeny2 - x->gl_screeny1);
@@ -557,7 +528,7 @@ static void canvas_setbounds(t_canvas *x, int x1, int y1, int x2, int y2)
             fix so that zero is bottom edge and redraw.  This is
             only appropriate if we're a regular "text" object on the
             parent. */
-        float diff = x->gl_y1 - x->gl_y2;
+        t_float diff = x->gl_y1 - x->gl_y2;
         t_gobj *y;
         x->gl_y1 = heightwas * diff;
         x->gl_y2 = x->gl_y1 - diff;
@@ -598,18 +569,21 @@ void canvas_reflecttitle(t_canvas *x)
         strcat(namebuf, ")");
     }
     else namebuf[0] = 0;
-    sys_vgui("wm title .x%lx {%s%c%s - %s}\n", 
-        x, x->gl_name->s_name, (x->gl_dirty? '*' : ' '), namebuf,
-            canvas_getdir(x)->s_name);
+    sys_vgui("pdtk_canvas_reflecttitle .x%lx {%s} {%s} {%s} %d\n",
+        x, canvas_getdir(x)->s_name, x->gl_name->s_name, namebuf, x->gl_dirty);
 }
 
-void canvas_dirty(t_canvas *x, t_int n)
+    /* mark a glist dirty or clean */
+void canvas_dirty(t_canvas *x, t_floatarg n)
 {
     t_canvas *x2 = canvas_getrootfor(x);
+    if (glist_amreloadingabstractions)
+        return;
     if ((unsigned)n != x2->gl_dirty)
     {
         x2->gl_dirty = n;
-        canvas_reflecttitle(x2);
+        if (x2->gl_havewindow)
+            canvas_reflecttitle(x2);
     }
 }
 
@@ -659,15 +633,8 @@ void canvas_map(t_canvas *x, t_floatarg f)
     {
         if (glist_isvisible(x))
         {
-                /* just clear out the whole canvas... */
+                /* just clear out the whole canvas */
             sys_vgui(".x%lx.c delete all\n", x);
-#if 0
-                /* alternatively, we could have erased them one by one...
-            for (y = x->gl_list; y; y = y->g_next)
-                gobj_vis(y, x, 0);
-                    ... but we should go through and erase the lines as well
-                    if we do it that way. */
-#endif
             x->gl_mapped = 0;
         }
     }
@@ -682,143 +649,6 @@ void canvas_redraw(t_canvas *x)
     }
 }
 
-/* ----  editors -- perhaps this and "vis" should go to g_editor.c ------- */
-
-static t_editor *editor_new(t_glist *owner)
-{
-    char buf[40];
-    t_editor *x = (t_editor *)getbytes(sizeof(*x));
-    x->e_connectbuf = binbuf_new();
-    x->e_deleted = binbuf_new();
-    x->e_glist = owner;
-    sprintf(buf, ".x%lx", (t_int)owner);
-    x->e_guiconnect = guiconnect_new(&owner->gl_pd, gensym(buf));
-    return (x);
-}
-
-static void editor_free(t_editor *x, t_glist *y)
-{
-    glist_noselect(y);
-    guiconnect_notarget(x->e_guiconnect, 1000);
-    binbuf_free(x->e_connectbuf);
-    binbuf_free(x->e_deleted);
-    freebytes((void *)x, sizeof(*x));
-}
-
-    /* recursively create or destroy all editors of a glist and its 
-    sub-glists, as long as they aren't toplevels. */
-void canvas_create_editor(t_glist *x, int createit)
-{
-    t_gobj *y;
-    t_object *ob;
-    if (createit)
-    {
-        if (x->gl_editor)
-            bug("canvas_create_editor");
-        else
-        {
-            x->gl_editor = editor_new(x);
-            for (y = x->gl_list; y; y = y->g_next)
-                if (ob = pd_checkobject(&y->g_pd))
-                    rtext_new(x, ob);
-        }
-    }
-    else
-    {
-        if (!x->gl_editor)
-            bug("canvas_create_editor");
-        else
-        {
-            for (y = x->gl_list; y; y = y->g_next)
-                if (ob = pd_checkobject(&y->g_pd))
-                    rtext_free(glist_findrtext(x, ob));
-            editor_free(x->gl_editor, x);
-            x->gl_editor = 0;
-        }
-    }
-    for (y = x->gl_list; y; y = y->g_next)
-        if (pd_class(&y->g_pd) == canvas_class &&
-            ((t_canvas *)y)->gl_isgraph)
-                canvas_create_editor((t_canvas *)y, createit);
-}
-
-    /* we call this when we want the window to become visible, mapped, and
-    in front of all windows; or with "f" zero, when we want to get rid of
-    the window. */
-void canvas_vis(t_canvas *x, t_floatarg f)
-{
-    char buf[30];
-    int flag = (f != 0);
-    if (flag)
-    {
-        /* post("havewindow %d, isgraph %d, isvisible %d  editor %d",
-            x->gl_havewindow, x->gl_isgraph, glist_isvisible(x),
-                (x->gl_editor != 0)); */
-            /* test if we're already visible and toplevel */
-        if (x->gl_editor)
-        {           /* just put us in front */
-#ifdef MSW
-            canvas_vis(x, 0);
-            canvas_vis(x, 1);
-#else
-            sys_vgui("raise .x%lx\n", x);
-            sys_vgui("focus .x%lx.c\n", x);
-            sys_vgui("wm deiconify .x%lx\n", x);  
-#endif
-        }
-        else
-        {
-            canvas_create_editor(x, 1);
-            sys_vgui("pdtk_canvas_new .x%lx %d %d +%d+%d %d\n", x,
-                (int)(x->gl_screenx2 - x->gl_screenx1),
-                (int)(x->gl_screeny2 - x->gl_screeny1),
-                (int)(x->gl_screenx1), (int)(x->gl_screeny1),
-                x->gl_edit);
-            canvas_reflecttitle(x);
-            x->gl_havewindow = 1;
-            canvas_updatewindowlist();
-        }
-    }
-    else    /* make invisible */
-    {
-        int i;
-        t_canvas *x2;
-        if (!x->gl_havewindow)
-        {
-                /* bug workaround -- a graph in a visible patch gets "invised"
-                when the patch is closed, and must lose the editor here.  It's
-                probably not the natural place to do this.  Other cases like
-                subpatches fall here too but don'd need the editor freed, so
-                we check if it exists. */
-            if (x->gl_editor)
-                canvas_create_editor(x, 0);
-            return;
-        }
-        sys_vgui("pdtk_canvas_getscroll .x%lx.c\n", x);
-        glist_noselect(x);
-        if (glist_isvisible(x))
-            canvas_map(x, 0);
-        canvas_create_editor(x, 0);
-        sys_vgui("destroy .x%lx\n", x);
-        for (i = 1, x2 = x; x2; x2 = x2->gl_next, i++)
-            ;
-        sys_vgui(".mbar.find delete %d\n", i);
-            /* if we're a graph on our parent, and if the parent exists
-               and is visible, show ourselves on parent. */
-        if (glist_isgraph(x) && x->gl_owner)
-        {
-            t_glist *gl2 = x->gl_owner;
-            canvas_create_editor(x, 1);
-            if (glist_isvisible(gl2))
-                gobj_vis(&x->gl_gobj, gl2, 0);
-            x->gl_havewindow = 0;
-            if (glist_isvisible(gl2))
-                gobj_vis(&x->gl_gobj, gl2, 1);
-        }
-        else x->gl_havewindow = 0;
-        canvas_updatewindowlist();
-    }
-}
 
     /* we call this on a non-toplevel glist to "open" it into its
     own window. */
@@ -828,13 +658,14 @@ void glist_menu_open(t_glist *x)
     {
         t_glist *gl2 = x->gl_owner;
         if (!gl2) 
-            bug("canvas_vis");  /* shouldn't happen but don't get too upset. */
+            bug("glist_menu_open");  /* shouldn't happen but not dangerous */
         else
         {
                 /* erase ourself in parent window */
             gobj_vis(&x->gl_gobj, gl2, 0);
                     /* get rid of our editor (and subeditors) */
-            canvas_create_editor(x, 0);
+            if (x->gl_editor)
+                canvas_destroy_editor(x);
             x->gl_havewindow = 1;
                     /* redraw ourself in parent window (blanked out this time) */
             gobj_vis(&x->gl_gobj, gl2, 1);
@@ -876,7 +707,8 @@ void canvas_free(t_canvas *x)
     glist_noselect(x);
     while (y = x->gl_list)
         glist_delete(x, y);
-    canvas_vis(x, 0);
+    if (x == glist_getcanvas(x))
+        canvas_vis(x, 0);
 
     if (strcmp(x->gl_name->s_name, "Pd"))
         pd_unbind(&x->gl_pd, canvas_makebindsym(x->gl_name));
@@ -901,7 +733,7 @@ static void canvas_drawlines(t_canvas *x)
     {
         linetraverser_start(&t, x);
         while (oc = linetraverser_next(&t))
-            sys_vgui(".x%lx.c create line %d %d %d %d -width %d -tags l%lx\n",
+            sys_vgui(".x%lx.c create line %d %d %d %d -width %d -tags [list l%lx cord]\n",
                     glist_getcanvas(x),
                         t.tr_lx1, t.tr_ly1, t.tr_lx2, t.tr_ly2, 
                             (outlet_getsymbol(t.tr_outlet) == &s_signal ? 2:1),
@@ -1043,19 +875,8 @@ void canvas_loadbang(t_canvas *x)
     canvas_loadbangsubpatches(x);
 }
 
-    /* When you ask a canvas its size the result is 2 pixels more than what
-    you gave it to open it; perhaps there's a 1-pixel border all around it
-    or something.  Anyway, we just add the 2 pixels back here; seems we
-    have to do this for linux but not MSW; not sure about MacOS. */
-
-#ifdef MSW
-#define HORIZBORDER 0
-#define VERTBORDER 0
-#else
-#define HORIZBORDER 2
-#define VERTBORDER 2
-#endif
-
+/* no longer used by 'pd-gui', but kept here for backwards compatibility.  The
+ * new method calls canvas_setbounds() directly. */
 static void canvas_relocate(t_canvas *x, t_symbol *canvasgeom,
     t_symbol *topgeom)
 {
@@ -1067,8 +888,8 @@ static void canvas_relocate(t_canvas *x, t_symbol *canvasgeom,
             /* for some reason this is initially called with cw=ch=1 so
             we just suppress that here. */
     if (cw > 5 && ch > 5)
-        canvas_setbounds(x, txpix, typix,
-            txpix + cw - HORIZBORDER, typix + ch - VERTBORDER);
+        canvas_dosetbounds(x, txpix, typix,
+            txpix + cw, typix + ch);
 }
 
 void canvas_popabstraction(t_canvas *x)
@@ -1471,6 +1292,22 @@ void canvas_savedeclarationsto(t_canvas *x, t_binbuf *b)
     }
 }
 
+static void canvas_completepath(char *from, char *to, int bufsize)
+{
+    if (sys_isabsolutepath(from))
+    {
+        to[0] = '\0';
+    }
+    else
+    {   // if not absolute path, append Pd lib dir
+        strncpy(to, sys_libdir->s_name, bufsize-4);
+        to[bufsize-3] = '\0';
+        strcat(to, "/");
+    }
+    strncat(to, from, bufsize-strlen(to));
+    to[bufsize-1] = '\0';
+}
+
 static void canvas_declare(t_canvas *x, t_symbol *s, int argc, t_atom *argv)
 {
     int i;
@@ -1492,12 +1329,8 @@ static void canvas_declare(t_canvas *x, t_symbol *s, int argc, t_atom *argv)
         }
         else if ((argc > i+1) && !strcmp(flag, "-stdpath"))
         {
-            strncpy(strbuf, sys_libdir->s_name, MAXPDSTRING-3);
-            strbuf[MAXPDSTRING-4] = 0;
-            strcat(strbuf, "/");
-            strncpy(strbuf, atom_getsymbolarg(i+1, argc, argv)->s_name,
-                MAXPDSTRING-strlen(strbuf));
-            strbuf[MAXPDSTRING-1] = 0;
+            canvas_completepath(atom_getsymbolarg(i+1, argc, argv)->s_name,
+                strbuf, MAXPDSTRING);
             e->ce_path = namelist_append(e->ce_path, strbuf, 0);
             i++;
         }
@@ -1508,12 +1341,8 @@ static void canvas_declare(t_canvas *x, t_symbol *s, int argc, t_atom *argv)
         }
         else if ((argc > i+1) && !strcmp(flag, "-stdlib"))
         {
-            strncpy(strbuf, sys_libdir->s_name, MAXPDSTRING-3);
-            strbuf[MAXPDSTRING-4] = 0;
-            strcat(strbuf, "/");
-            strncpy(strbuf, atom_getsymbolarg(i+1, argc, argv)->s_name,
-                MAXPDSTRING-strlen(strbuf));
-            strbuf[MAXPDSTRING-1] = 0;
+            canvas_completepath(atom_getsymbolarg(i+1, argc, argv)->s_name,
+                strbuf, MAXPDSTRING);
             sys_load_lib(0, strbuf);
             i++;
         }
@@ -1560,9 +1389,16 @@ int canvas_open(t_canvas *x, const char *name, const char *ext,
         for (nl = y->gl_env->ce_path; nl; nl = nl->nl_next)
         {
             char realname[MAXPDSTRING];
-            strncpy(realname, dir, MAXPDSTRING);
-            realname[MAXPDSTRING-3] = 0;
-            strcat(realname, "/");
+            if (sys_isabsolutepath(nl->nl_string))
+            {
+                realname[0] = '\0';
+            }
+            else
+            {   /* if not absolute path, append Pd lib dir */
+                strncpy(realname, dir, MAXPDSTRING);
+                realname[MAXPDSTRING-3] = 0;
+                strcat(realname, "/");
+            }
             strncat(realname, nl->nl_string, MAXPDSTRING-strlen(realname));
             realname[MAXPDSTRING-1] = 0;
             if ((fd = sys_trytoopenone(realname, name, ext,
@@ -1664,6 +1500,8 @@ void g_canvas_setup(void)
         A_DEFFLOAT, A_NULL);
     class_addmethod(canvas_class, (t_method)canvas_loadbang,
         gensym("loadbang"), A_NULL);
+    class_addmethod(canvas_class, (t_method)canvas_setbounds,
+        gensym("setbounds"), A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_NULL);
     class_addmethod(canvas_class, (t_method)canvas_relocate,
         gensym("relocate"), A_SYMBOL, A_SYMBOL, A_NULL);
     class_addmethod(canvas_class, (t_method)canvas_vis,
@@ -1672,6 +1510,8 @@ void g_canvas_setup(void)
         gensym("menu-open"), A_NULL);
     class_addmethod(canvas_class, (t_method)canvas_map,
         gensym("map"), A_FLOAT, A_NULL);
+    class_addmethod(canvas_class, (t_method)canvas_dirty,
+        gensym("dirty"), A_FLOAT, A_NULL);
     class_setpropertiesfn(canvas_class, (t_propertiesfn)canvas_properties);
 
 /* ---------------------- list handling ------------------------ */
