@@ -8,7 +8,11 @@
 #include "m_imp.h"
 #include "s_stuff.h"
 #include "g_canvas.h"
+#include "s_utf8.h" /*-- moo --*/
 #include <string.h>
+#ifdef _MSC_VER  /* This is only for Microsoft's compiler, not cygwin, e.g. */
+#define snprintf sprintf_s
+#endif
 
 void glist_readfrombinbuf(t_glist *x, t_binbuf *b, char *filename,
     int selectem);
@@ -27,6 +31,7 @@ static void canvas_clearline(t_canvas *x);
 static t_binbuf *copy_binbuf;
 static char *canvas_textcopybuf;
 static int canvas_textcopybufsize;
+static t_glist *glist_finddirty(t_glist *x);
 
 /* ---------------- generic widget behavior ------------------------- */
 
@@ -61,28 +66,44 @@ void gobj_delete(t_gobj *x, t_glist *glist)
         (*x->g_pd->c_wb->w_deletefn)(x, glist);
 }
 
+int gobj_shouldvis(t_gobj *x, struct _glist *glist)
+{
+    t_object *ob;
+    if (!glist->gl_havewindow && glist->gl_isgraph && glist->gl_goprect &&
+        glist->gl_owner && (pd_class(&x->g_pd) != scalar_class)
+            && (pd_class(&x->g_pd) != garray_class))
+    {
+        /* if we're graphing-on-parent and the object falls outside the
+        graph rectangle, don't draw it. */
+        int x1, y1, x2, y2, gx1, gy1, gx2, gy2, m;
+        gobj_getrect(&glist->gl_gobj, glist->gl_owner, &x1, &y1, &x2, &y2);
+        if (x1 > x2)
+            m = x1, x1 = x2, x2 = m;
+        if (y1 > y2)
+            m = y1, y1 = y2, y2 = m;
+        gobj_getrect(x, glist, &gx1, &gy1, &gx2, &gy2);
+        if (gx1 < x1 || gx1 > x2 || gx2 < x1 || gx2 > x2 ||
+            gy1 < y1 || gy1 > y2 || gy2 < y1 || gy2 > y2)
+                return (0);
+    }
+    if (ob = pd_checkobject(&x->g_pd))
+    {
+        /* return true if the text box should be drawn.  We don't show text
+        boxes inside graphs---except comments, if we're doing the new
+        (goprect) style. */
+        return (glist->gl_havewindow ||
+            (ob->te_pd != canvas_class &&
+                ob->te_pd->c_wb != &text_widgetbehavior) ||
+            (ob->te_pd == canvas_class && (((t_glist *)ob)->gl_isgraph)) ||
+            (glist->gl_goprect && (ob->te_type == T_TEXT)));
+    }
+    else return (1);
+}
+
 void gobj_vis(t_gobj *x, struct _glist *glist, int flag)
 {
-    if (x->g_pd->c_wb && x->g_pd->c_wb->w_visfn)
-    {
-        if (!glist->gl_havewindow && glist->gl_isgraph && glist->gl_goprect &&
-            glist->gl_owner && (pd_class(&glist->gl_pd) != garray_class))
-        {
-            /* if we're graphing-on-parent and the object falls outside the
-            graph rectangle, don't draw it. */
-            int x1, y1, x2, y2, gx1, gy1, gx2, gy2, m;
-            gobj_getrect(&glist->gl_gobj, glist->gl_owner, &x1, &y1, &x2, &y2);
-            if (x1 > x2)
-                m = x1, x1 = x2, x2 = m;
-            if (y1 > y2)
-                m = y1, y1 = y2, y2 = m;
-            gobj_getrect(x, glist, &gx1, &gy1, &gx2, &gy2);
-            if (gx1 < x1 || gx1 > x2 || gx2 < x1 || gx2 > x2 ||
-                gy1 < y1 || gy1 > y2 || gy2 < y1 || gy2 > y2)
-                    return;
-        }
+    if (x->g_pd->c_wb && x->g_pd->c_wb->w_visfn && gobj_shouldvis(x, glist))
         (*x->g_pd->c_wb->w_visfn)(x, glist, flag);
-    }
 }
 
 int gobj_click(t_gobj *x, struct _glist *glist,
@@ -151,12 +172,26 @@ void glist_select(t_glist *x, t_gobj *y)
     }
 }
 
+    /* recursively deselect everything in a gobj "g", if it happens to be
+    a glist, in preparation for deselecting g itself in glist_dselect() */
+static void glist_checkanddeselectall(t_glist *gl, t_gobj *g)
+{
+    t_glist *gl2;
+    t_gobj *g2;
+    if (pd_class(&g->g_pd) != canvas_class)
+        return;
+    gl2 = (t_glist *)g;
+    for (g2 = gl2->gl_list; g2; g2 = g2->g_next)
+        glist_checkanddeselectall(gl2, g2);
+    glist_noselect(gl2);
+}
+
     /* call this for selected objects only */
 void glist_deselect(t_glist *x, t_gobj *y)
 {
     int fixdsp = 0;
     static int reenter = 0;
-    if (reenter) return;
+    /* if (reenter) return; */
     reenter = 1;
     if (x->gl_editor)
     {
@@ -172,6 +207,7 @@ void glist_deselect(t_glist *x, t_gobj *y)
                 {
                     z = fuddy;
                     canvas_stowconnections(glist_getcanvas(x));
+                    glist_checkanddeselectall(x, y);
                 }
                 gobj_activate(y, x, 0);
             }
@@ -205,7 +241,7 @@ void glist_deselect(t_glist *x, t_gobj *y)
 
             rtext_gettext(z, &buf, &bufsize);
             text_setto((t_text *)y, x, buf, bufsize);
-            canvas_fixlinesfor(glist_getcanvas(x), (t_text *)y);
+            canvas_fixlinesfor(x, (t_text *)y);
             x->gl_editor->e_textedfor = 0;
         }
         if (fixdsp)
@@ -682,6 +718,7 @@ static void glist_doreload(t_glist *gl, t_symbol *name, t_symbol *dir,
 {
     t_gobj *g;
     int i, nobj = glist_getindex(gl, 0);  /* number of objects */
+    int hadwindow = (gl->gl_editor != 0);
     for (g = gl->gl_list, i = 0; g && i < nobj; i++)
     {
         if (g != except && pd_class(&g->g_pd) == canvas_class &&
@@ -694,9 +731,10 @@ static void glist_doreload(t_glist *gl, t_symbol *name, t_symbol *dir,
                 replacement will be at the end of the list, so we don't
                 do g = g->g_next in this case. */
             int j = glist_getindex(gl, g);
-            int hadwindow = gl->gl_havewindow;
-            if (!hadwindow)
-                canvas_vis(glist_getcanvas(gl), 1);
+            if (!gl->gl_editor)
+                canvas_vis(gl, 1);
+            if (!gl->gl_editor)
+                bug("editor");
             glist_noselect(gl);
             glist_select(gl, g);
             canvas_setundo(gl, canvas_undo_cut,
@@ -705,8 +743,6 @@ static void glist_doreload(t_glist *gl, t_symbol *name, t_symbol *dir,
             canvas_undo(gl);
             glist_noselect(gl);
             g = glist_nth(gl, j);
-            if (!hadwindow)
-                canvas_vis(glist_getcanvas(gl), 0);
         }
         else
         {
@@ -715,31 +751,37 @@ static void glist_doreload(t_glist *gl, t_symbol *name, t_symbol *dir,
              g = g->g_next;
         }
     }
+    if (!hadwindow && gl->gl_editor)
+        canvas_vis(glist_getcanvas(gl), 0);
 }
 
+    /* this flag stops canvases from being marked "dirty" if we have to touch
+    them to reload an abstraction; also suppress window list update */
+int glist_amreloadingabstractions = 0;
+
     /* call canvas_doreload on everyone */
 void canvas_reload(t_symbol *name, t_symbol *dir, t_gobj *except)
 {
     t_canvas *x;
+    int dspwas = canvas_suspend_dsp();
+    glist_amreloadingabstractions = 1;
         /* find all root canvases */
     for (x = canvas_list; x; x = x->gl_next)
         glist_doreload(x, name, dir, except);
+    glist_amreloadingabstractions = 0;
+    canvas_resume_dsp(dspwas);
 }
 
 /* ------------------------ event handling ------------------------ */
 
 static char *cursorlist[] = {
-#ifdef MSW
-    "right_ptr",        /* CURSOR_RUNMODE_NOTHING */
-#else
-    "left_ptr",         /* CURSOR_RUNMODE_NOTHING */
-#endif
-    "arrow",            /* CURSOR_RUNMODE_CLICKME */
-    "sb_v_double_arrow", /* CURSOR_RUNMODE_THICKEN */
-    "plus",             /* CURSOR_RUNMODE_ADDPOINT */
-    "hand2",            /* CURSOR_EDITMODE_NOTHING */
-    "circle",           /* CURSOR_EDITMODE_CONNECT */
-    "X_cursor"          /* CURSOR_EDITMODE_DISCONNECT */
+    "$cursor_runmode_nothing",
+    "$cursor_runmode_clickme",
+    "$cursor_runmode_thicken",
+    "$cursor_runmode_addpoint",
+    "$cursor_editmode_nothing",
+    "$cursor_editmode_connect",
+    "$cursor_editmode_disconnect"
 };
 
 void canvas_setcursor(t_canvas *x, unsigned int cursornum)
@@ -765,9 +807,8 @@ int canvas_hitbox(t_canvas *x, t_gobj *y, int xpos, int ypos,
 {
     int x1, y1, x2, y2;
     t_text *ob;
-    if ((ob = pd_checkobject(&y->g_pd)) && 
-        !text_shouldvis(ob, x))
-            return (0);
+    if (!gobj_shouldvis(y, x))
+        return (0);
     gobj_getrect(y, x, &x1, &y1, &x2, &y2);
     if (xpos >= x1 && xpos <= x2 && ypos >= y1 && ypos <= y2)
     {
@@ -817,12 +858,156 @@ static void canvas_rightclick(t_canvas *x, int xpos, int ypos, t_gobj *y)
         x, xpos, ypos, canprop, canopen);
 }
 
+/* ----  editors -- perhaps this and "vis" should go to g_editor.c ------- */
+
+static t_editor *editor_new(t_glist *owner)
+{
+    char buf[40];
+    t_editor *x = (t_editor *)getbytes(sizeof(*x));
+    x->e_connectbuf = binbuf_new();
+    x->e_deleted = binbuf_new();
+    x->e_glist = owner;
+    sprintf(buf, ".x%lx", (t_int)owner);
+    x->e_guiconnect = guiconnect_new(&owner->gl_pd, gensym(buf));
+    x->e_clock = 0;
+    return (x);
+}
+
+static void editor_free(t_editor *x, t_glist *y)
+{
+    glist_noselect(y);
+    guiconnect_notarget(x->e_guiconnect, 1000);
+    binbuf_free(x->e_connectbuf);
+    binbuf_free(x->e_deleted);
+    if (x->e_clock)
+        clock_free(x->e_clock);
+    freebytes((void *)x, sizeof(*x));
+}
+
+    /* recursively create or destroy all editors of a glist and its 
+    sub-glists, as long as they aren't toplevels. */
+void canvas_create_editor(t_glist *x)
+{
+    t_gobj *y;
+    t_object *ob;
+    if (!x->gl_editor)
+    {
+        x->gl_editor = editor_new(x);
+        for (y = x->gl_list; y; y = y->g_next)
+            if (ob = pd_checkobject(&y->g_pd))
+                rtext_new(x, ob);
+    }
+}
+
+void canvas_destroy_editor(t_glist *x)
+{
+    t_gobj *y;
+    t_object *ob;
+    glist_noselect(x);
+    if (x->gl_editor)
+    {
+        for (y = x->gl_list; y; y = y->g_next)
+            if (ob = pd_checkobject(&y->g_pd))
+                rtext_free(glist_findrtext(x, ob));
+        editor_free(x->gl_editor, x);
+        x->gl_editor = 0;
+    }
+}
+
+void canvas_reflecttitle(t_canvas *x);
+void canvas_map(t_canvas *x, t_floatarg f);
+
+    /* we call this when we want the window to become visible, mapped, and
+    in front of all windows; or with "f" zero, when we want to get rid of
+    the window. */
+void canvas_vis(t_canvas *x, t_floatarg f)
+{
+    char buf[30];
+    int flag = (f != 0);
+    if (flag)
+    {
+        /* If a subpatch/abstraction has GOP/gl_isgraph set, then it will have
+         * a gl_editor already, if its not, it will not have a gl_editor.
+         * canvas_create_editor(x) checks if a gl_editor is already created,
+         * so its ok to run it on a canvas that already has a gl_editor. */
+        if (x->gl_editor && x->gl_havewindow)
+        {           /* just put us in front */
+            sys_vgui("pdtk_canvas_raise .x%lx\n", x);  
+        }
+        else
+        {
+            char cbuf[MAXPDSTRING];
+            int cbuflen;
+            t_canvas *c = x;
+            canvas_create_editor(x);
+            sys_vgui("pdtk_canvas_new .x%lx %d %d +%d+%d %d\n", x,
+                (int)(x->gl_screenx2 - x->gl_screenx1),
+                (int)(x->gl_screeny2 - x->gl_screeny1),
+                (int)(x->gl_screenx1), (int)(x->gl_screeny1),
+                x->gl_edit);
+            snprintf(cbuf, MAXPDSTRING - 2, "pdtk_canvas_setparents .x%lx",
+                (unsigned long)c);
+            while (c->gl_owner) {
+                c = c->gl_owner;
+                cbuflen = strlen(cbuf);
+                snprintf(cbuf + cbuflen,
+                         MAXPDSTRING - cbuflen - 2,/* leave 2 for "\n\0" */
+                         " .x%lx", (unsigned long)c);
+            }
+            strcat(cbuf, "\n");
+            sys_gui(cbuf);
+            canvas_reflecttitle(x);
+            x->gl_havewindow = 1;
+            canvas_updatewindowlist();
+        }
+    }
+    else    /* make invisible */
+    {
+        int i;
+        t_canvas *x2;
+        if (!x->gl_havewindow)
+        {
+                /* bug workaround -- a graph in a visible patch gets "invised"
+                when the patch is closed, and must lose the editor here.  It's
+                probably not the natural place to do this.  Other cases like
+                subpatches fall here too but don'd need the editor freed, so
+                we check if it exists. */
+            if (x->gl_editor)
+                canvas_destroy_editor(x);
+            return;
+        }
+        glist_noselect(x);
+        if (glist_isvisible(x))
+            canvas_map(x, 0);
+        canvas_destroy_editor(x);
+        sys_vgui("destroy .x%lx\n", x);
+        for (i = 1, x2 = x; x2; x2 = x2->gl_next, i++)
+            ;
+            /* if we're a graph on our parent, and if the parent exists
+               and is visible, show ourselves on parent. */
+        if (glist_isgraph(x) && x->gl_owner)
+        {
+            t_glist *gl2 = x->gl_owner;
+            if (glist_isvisible(gl2))
+                gobj_vis(&x->gl_gobj, gl2, 0);
+            x->gl_havewindow = 0;
+            if (glist_isvisible(gl2))
+                gobj_vis(&x->gl_gobj, gl2, 1);
+        }
+        else x->gl_havewindow = 0;
+        canvas_updatewindowlist();
+    }
+}
+
     /* set a canvas up as a graph-on-parent.  Set reasonable defaults for
     any missing paramters and redraw things if necessary. */
 void canvas_setgraph(t_glist *x, int flag, int nogoprect)
 {
     if (!flag && glist_isgraph(x))
     {
+        int hadeditor = (x->gl_editor != 0);
+        if (hadeditor)
+            canvas_destroy_editor(x);
         if (x->gl_owner && !x->gl_loading && glist_isvisible(x->gl_owner))
             gobj_vis(&x->gl_gobj, x->gl_owner, 0);
         x->gl_isgraph = 0;
@@ -844,20 +1029,9 @@ void canvas_setgraph(t_glist *x, int flag, int nogoprect)
             gobj_vis(&x->gl_gobj, x->gl_owner, 0);
         x->gl_isgraph = 1;
         x->gl_hidetext = !(!(flag&2));
-        if (!nogoprect && !x->gl_goprect)
-        {
-            t_gobj *g;
-            for (g = x->gl_list; g; g = g->g_next)
-                if (pd_checkobject(&g->g_pd))
-            {
-                x->gl_goprect = 1;
-                break;
-            }
-        }
+        x->gl_goprect = !nogoprect;
         if (glist_isvisible(x) && x->gl_goprect)
             glist_redraw(x);
-        if (x->gl_loading && x->gl_owner && glist_isvisible(x->gl_owner))
-            canvas_create_editor(x, 1);
         if (x->gl_owner && !x->gl_loading && glist_isvisible(x->gl_owner))
         {
             gobj_vis(&x->gl_gobj, x->gl_owner, 1);
@@ -966,6 +1140,7 @@ static void canvas_donecanvasdialog(t_glist *x,
     }
         /* LATER avoid doing 2 redraws here (possibly one inside setgraph) */
     canvas_setgraph(x, graphme, 0);
+    canvas_dirty(x, 1);
     if (x->gl_havewindow)
         canvas_redraw(x);
     else if (glist_isvisible(x->gl_owner))
@@ -1019,8 +1194,9 @@ static void canvas_done_popup(t_canvas *x, t_float which, t_float xpos, t_float
                     strcpy(namebuf, class_gethelpname(pd_class(&y->g_pd)));
                     dir = class_gethelpdir(pd_class(&y->g_pd));
                 }
-                if (strcmp(namebuf + strlen(namebuf) - 3, ".pd"))
-                    strcat(namebuf, ".pd");
+                if (strlen(namebuf) < 4 ||
+                    strcmp(namebuf + strlen(namebuf) - 3, ".pd"))
+                        strcat(namebuf, ".pd");
                 open_via_helppath(namebuf, dir);
                 return;
             }
@@ -1247,7 +1423,7 @@ void canvas_doclick(t_canvas *x, int xpos, int ypos, int which,
     }
 }
 
-void canvas_mousedown(t_canvas *x, t_floatarg xpos, t_floatarg ypos,
+void canvas_mouse(t_canvas *x, t_floatarg xpos, t_floatarg ypos,
     t_floatarg which, t_floatarg mod)
 {
     canvas_doclick(x, xpos, ypos, which, mod, 1);
@@ -1339,10 +1515,11 @@ void canvas_doconnect(t_canvas *x, int xpos, int ypos, int which, int doit)
                         ((x22-x21-IOWIDTH) * closest2)/(ninlet2-1) : 0)
                             + IOMIDDLE;
                 ly2 = y21;
-                sys_vgui(".x%lx.c create line %d %d %d %d -width %d -tags l%lx\n",
+                sys_vgui(".x%lx.c create line %d %d %d %d -width %d -tags [list l%lx cord]\n",
                     glist_getcanvas(x),
                         lx1, ly1, lx2, ly2,
                             (obj_issignaloutlet(ob1, closest1) ? 2 : 1), oc);
+                canvas_dirty(x, 1);
                 canvas_setundo(x, canvas_undo_connect,
                     canvas_undo_set_connect(x, 
                         canvas_getindex(x, &ob1->ob_g), closest1,
@@ -1382,7 +1559,7 @@ static void canvas_doregion(t_canvas *x, int xpos, int ypos, int doit)
         else hiy = x->gl_editor->e_ywas, loy = ypos;
         canvas_selectinrect(x, lox, loy, hix, hiy);
         sys_vgui(".x%lx.c delete x\n", x);
-        x->gl_editor->e_onmotion = 0;
+        x->gl_editor->e_onmotion = MA_NONE;
     }
     else sys_vgui(".x%lx.c coords x %d %d %d %d\n",
             x, x->gl_editor->e_xwas,
@@ -1413,11 +1590,27 @@ void canvas_mouseup(t_canvas *x,
             /* after motion, if there's only one item selected, activate it */
         if (x->gl_editor->e_selection &&
             !(x->gl_editor->e_selection->sel_next))
-                gobj_activate(x->gl_editor->e_selection->sel_what,
-                    x, 1);
+        {
+            t_gobj *g = x->gl_editor->e_selection->sel_what;
+            t_glist *gl2;
+                /* first though, check we aren't an abstraction with a
+                dirty sub-patch that would be discarded if we edit this. */
+            if (pd_class(&g->g_pd) == canvas_class &&
+                canvas_isabstraction((t_glist *)g) &&
+                    (gl2 = glist_finddirty((t_glist *)g)))
+            {
+                vmess(&gl2->gl_pd, gensym("menu-open"), "");
+                x->gl_editor->e_onmotion = MA_NONE;
+                sys_vgui(
+"pdtk_check .x%lx {Discard changes to '%s'?} {.x%lx dirty 0;\n} no\n",
+                    canvas_getrootfor(gl2),
+                        canvas_getrootfor(gl2)->gl_name->s_name, gl2);
+                return;
+            }
+                /* OK, activate it */
+            gobj_activate(x->gl_editor->e_selection->sel_what, x, 1);
+        }
     }
-    if (x->gl_editor->e_onmotion != MA_NONE)
-        sys_vgui("pdtk_canvas_getscroll .x%lx.c\n", x);
 
     x->gl_editor->e_onmotion = MA_NONE;
 }
@@ -1442,7 +1635,9 @@ static void canvas_displaceselection(t_canvas *x, int dx, int dy)
     }
     if (resortin) canvas_resortinlets(x);
     if (resortout) canvas_resortoutlets(x);
-    canvas_dirty(x, 1);
+    sys_vgui("pdtk_canvas_getscroll .x%lx.c\n", x);
+    if (x->gl_editor->e_selection)
+        canvas_dirty(x, 1);
 }
 
     /* this routine is called whenever a key is pressed or released.  "x"
@@ -1468,9 +1663,20 @@ void canvas_key(t_canvas *x, t_symbol *s, int ac, t_atom *av)
         gotkeysym = av[1].a_w.w_symbol;
     else if (av[1].a_type == A_FLOAT)
     {
-        char buf[3];
-        sprintf(buf, "%c", (int)(av[1].a_w.w_float));
-        gotkeysym = gensym(buf);
+        char buf[UTF8_MAXBYTES1];
+        switch((int)(av[1].a_w.w_float))
+        {
+        case 8:  gotkeysym = gensym("BackSpace"); break;
+        case 9:  gotkeysym = gensym("Tab"); break;
+        case 10: gotkeysym = gensym("Return"); break;
+        case 27: gotkeysym = gensym("Escape"); break;
+        case 32: gotkeysym = gensym("Space"); break;
+        case 127:gotkeysym = gensym("Delete"); break;
+        default:
+        /*-- moo: assume keynum is a Unicode codepoint; encode as UTF-8 --*/
+            u8_wc_toutf8_nul(buf, (UCS4)(av[1].a_w.w_float));
+            gotkeysym = gensym(buf);
+        }
     }
     else gotkeysym = gensym("?");
     fflag = (av[0].a_type == A_FLOAT ? av[0].a_w.w_float : 0);
@@ -1519,6 +1725,10 @@ void canvas_key(t_canvas *x, t_symbol *s, int ac, t_atom *av)
         return;
     if (x && down)
     {
+        t_object *ob;
+            /* cancel any dragging action */
+        if (x->gl_editor->e_onmotion == MA_MOVE)
+            x->gl_editor->e_onmotion = MA_NONE;
             /* if an object has "grabbed" keys just send them on */
         if (x->gl_editor->e_grab
             && x->gl_editor->e_keyfn && keynum)
@@ -1532,6 +1742,7 @@ void canvas_key(t_canvas *x, t_symbol *s, int ac, t_atom *av)
             || !strcmp(gotkeysym->s_name, "Left")
             || !strcmp(gotkeysym->s_name, "Right")))
         {
+                /* send the key to the box's editor */
             if (!x->gl_editor->e_textdirty)
             {
                 canvas_setundo(x, canvas_undo_cut,
@@ -1572,6 +1783,15 @@ void canvas_key(t_canvas *x, t_symbol *s, int ac, t_atom *av)
                 CURSOR_RUNMODE_NOTHING :CURSOR_EDITMODE_NOTHING);
 }
 
+static void delay_move(t_canvas *x)
+{
+    canvas_displaceselection(x, 
+       x->gl_editor->e_xnew - x->gl_editor->e_xwas,
+       x->gl_editor->e_ynew - x->gl_editor->e_ywas);
+    x->gl_editor->e_xwas = x->gl_editor->e_xnew;
+    x->gl_editor->e_ywas = x->gl_editor->e_ynew;
+}
+
 void canvas_motion(t_canvas *x, t_floatarg xpos, t_floatarg ypos,
     t_floatarg fmod)
 { 
@@ -1585,10 +1805,12 @@ void canvas_motion(t_canvas *x, t_floatarg xpos, t_floatarg ypos,
     glist_setlastxy(x, xpos, ypos);
     if (x->gl_editor->e_onmotion == MA_MOVE)
     {
-        canvas_displaceselection(x, 
-            xpos - x->gl_editor->e_xwas, ypos - x->gl_editor->e_ywas);
-        x->gl_editor->e_xwas = xpos;
-        x->gl_editor->e_ywas = ypos;    
+        if (!x->gl_editor->e_clock)
+            x->gl_editor->e_clock = clock_new(x, (t_method)delay_move);
+        clock_unset(x->gl_editor->e_clock);
+        clock_delay(x->gl_editor->e_clock, 5);
+        x->gl_editor->e_xnew = xpos;
+        x->gl_editor->e_ynew = ypos;
     }
     else if (x->gl_editor->e_onmotion == MA_REGION)
         canvas_doregion(x, xpos, ypos, 0);
@@ -1628,6 +1850,7 @@ void canvas_startmotion(t_canvas *x)
 }
 
 /* ----------------------------- window stuff ----------------------- */
+extern int sys_perf;
 
 void canvas_print(t_canvas *x, t_symbol *s)
 {
@@ -1659,13 +1882,12 @@ void glob_verifyquit(void *dummy, t_floatarg f)
         if (g2 = glist_finddirty(g))
     {
         canvas_vis(g2, 1);
-        sys_vgui(
-"pdtk_check {Discard changes to this window??} {.x%lx menuclose 3;\n} no\n",
-                g2);
+            sys_vgui("pdtk_canvas_menuclose .x%lx {.x%lx menuclose 3;\n}\n",
+                     canvas_getrootfor(g2), g2);
         return;
     }
-    if (f == 0)
-        sys_vgui("pdtk_check {really quit?} {pd quit;\n} yes\n");
+    if (f == 0 && sys_perf)
+        sys_vgui("pdtk_check .pdwindow {really quit?} {pd quit} yes\n");
     else glob_quit(0);
 }
 
@@ -1687,18 +1909,17 @@ void canvas_menuclose(t_canvas *x, t_floatarg fforce)
         g = glist_finddirty(x);
         if (g)
         {
-            canvas_vis(g, 1);
-            sys_vgui(
-"pdtk_check {Discard changes to this window??} {.x%lx menuclose 2;\n} no\n",
-                g);
+            vmess(&g->gl_pd, gensym("menu-open"), "");
+            sys_vgui("pdtk_canvas_menuclose .x%lx {.x%lx menuclose 2;\n}\n",
+                     canvas_getrootfor(g), g);
             return;
         }
-        else
+        else if (sys_perf)
         {
-            sys_vgui(
-"pdtk_check {Close this window??} {.x%lx menuclose 1;\n} yes\n",
-                x);
+            sys_vgui("pdtk_canvas_menuclose .x%lx {.x%lx menuclose 1;\n}\n",
+                     canvas_getrootfor(g), g);
         }
+        else pd_free(&x->gl_pd);
     }
     else if (force == 1)
         pd_free(&x->gl_pd);
@@ -1710,10 +1931,9 @@ void canvas_menuclose(t_canvas *x, t_floatarg fforce)
         g = glist_finddirty(x);
         if (g)
         {
-            canvas_vis(g, 1);
-            sys_vgui(
-"pdtk_check {Discard changes to this window??} {.x%lx menuclose 2;\n} no\n",
-                g);
+            vmess(&g->gl_pd, gensym("menu-open"), "");
+            sys_vgui("pdtk_canvas_menuclose .x%lx {.x%lx menuclose 2;\n}\n",
+                     canvas_getrootfor(x), g);
             return;
         }
         else pd_free(&x->gl_pd);
@@ -1735,9 +1955,9 @@ static void canvas_menufont(t_canvas *x)
     gfxstub_new(&x2->gl_pd, &x2->gl_pd, buf);
 }
 
-static int canvas_find_index1, canvas_find_index2;
+static int canvas_find_index1, canvas_find_index2, canvas_find_wholeword;
 static t_binbuf *canvas_findbuf;
-int binbuf_match(t_binbuf *inbuf, t_binbuf *searchbuf);
+int binbuf_match(t_binbuf *inbuf, t_binbuf *searchbuf, int wholeword);
 
     /* find an atom or string of atoms */
 static int canvas_dofind(t_canvas *x, int *myindex1p)
@@ -1752,7 +1972,8 @@ static int canvas_dofind(t_canvas *x, int *myindex1p)
             t_object *ob = 0;
             if (ob = pd_checkobject(&y->g_pd))
             {
-                if (binbuf_match(ob->ob_binbuf, canvas_findbuf))
+                if (binbuf_match(ob->ob_binbuf, canvas_findbuf,
+                    canvas_find_wholeword))
                 {
                     if (myindex1 > canvas_find_index1 ||
                         myindex1 == canvas_find_index1 &&
@@ -1782,30 +2003,21 @@ static int canvas_dofind(t_canvas *x, int *myindex1p)
     return (0);
 }
 
-static void canvas_find(t_canvas *x, t_symbol *s, int ac, t_atom *av)
+static void canvas_find(t_canvas *x, t_symbol *s, t_floatarg wholeword)
 {
-    int myindex1 = 0, i;
-    for (i = 0; i < ac; i++)
-    {
-        if (av[i].a_type == A_SYMBOL)
-        {
-            if (!strcmp(av[i].a_w.w_symbol->s_name, "_semi_"))
-                SETSEMI(&av[i]);
-            else if (!strcmp(av[i].a_w.w_symbol->s_name, "_comma_"))
-                SETCOMMA(&av[i]);
-        }
-    }
+    int myindex1 = 0;
+    t_symbol *decodedsym = sys_decodedialog(s);
     if (!canvas_findbuf)
         canvas_findbuf = binbuf_new();
-    binbuf_clear(canvas_findbuf);
-    binbuf_add(canvas_findbuf, ac, av);
+    binbuf_text(canvas_findbuf, decodedsym->s_name, strlen(decodedsym->s_name));
     canvas_find_index1 = 0;
     canvas_find_index2 = -1;
+    canvas_find_wholeword = wholeword;
     canvas_whichfind = x;
     if (!canvas_dofind(x, &myindex1))
     {
         binbuf_print(canvas_findbuf);
-        post("... couldn't find");
+        sys_vgui("pdtk_couldnotfind .x%lx\n", x);
     }
 }
 
@@ -1817,7 +2029,7 @@ static void canvas_find_again(t_canvas *x)
     if (!canvas_dofind(canvas_whichfind, &myindex1))
     {
         binbuf_print(canvas_findbuf);
-        post("... couldn't find");
+        sys_vgui("pdtk_couldnotfind .x%lx\n", x);
     }
 }
 
@@ -1859,7 +2071,7 @@ void canvas_finderror(void *error_object)
         if (glist_dofinderror(x, error_object))
             return;
     }
-    post("... sorry, I couldn't find the source of that error.");
+    error("... sorry, I couldn't find the source of that error.");
 }
 
 void canvas_stowconnections(t_canvas *x)
@@ -1963,22 +2175,8 @@ static void canvas_copy(t_canvas *x)
         char *buf;
         int bufsize;
         rtext_getseltext(x->gl_editor->e_textedfor, &buf, &bufsize);
-
-#if defined(MSW) || defined(__APPLE__)
-            /* for Mac or Windows, copy the text to the clipboard here */
-        sys_vgui("clipboard clear\n", bufsize, buf);
+        sys_gui("clipboard clear\n");
         sys_vgui("clipboard append {%.*s}\n", bufsize, buf);
-#else
-            /* in X windows the selection already went to the
-            clipboard when it was made; here we "copy" it to our own buffer
-            as well, because, annoyingly, the clipboard will usually be 
-            destroyed by the time the user asks to "paste". */
-        if (canvas_textcopybuf)
-            t_freebytes(canvas_textcopybuf, canvas_textcopybufsize);
-        canvas_textcopybuf = (char *)getbytes(bufsize);
-        memcpy(canvas_textcopybuf, buf, bufsize);
-        canvas_textcopybufsize = bufsize;
-#endif
     }
 }
 
@@ -1990,6 +2188,7 @@ static void canvas_clearline(t_canvas *x)
              x->gl_editor->e_selectline_outno,
              x->gl_editor->e_selectline_index2,
              x->gl_editor->e_selectline_inno);
+        canvas_dirty(x, 1);
         canvas_setundo(x, canvas_undo_disconnect,
             canvas_undo_set_disconnect(x,
                 x->gl_editor->e_selectline_index1,
@@ -2059,6 +2258,8 @@ restore:
 
 static void canvas_cut(t_canvas *x)
 {
+    if (!x->gl_editor)  /* ignore if invisible */ 
+        return;
     if (x->gl_editor && x->gl_editor->e_selectedline)
         canvas_clearline(x);
     else if (x->gl_editor->e_textedfor)
@@ -2127,19 +2328,8 @@ static void canvas_paste(t_canvas *x)
         return;
     if (x->gl_editor->e_textedfor)
     {
-            /* simulate keystrokes as if the copy buffer were typed in. */
-#if defined(MSW) || defined(__APPLE__)
-            /* for Mac or Windows,  ask the GUI to send the clipboard down */
+        /* simulate keystrokes as if the copy buffer were typed in. */
         sys_gui("pdtk_pastetext\n");
-#else
-            /* in X windows we kept the text in our own copy buffer */
-        int i;
-        for (i = 0; i < canvas_textcopybufsize; i++)
-        {
-            pd_vmess(&x->gl_gobj.g_pd, gensym("key"), "iii",
-                1, canvas_textcopybuf[i]&0xff, 0);
-        }
-#endif
     }
     else
     {
@@ -2151,7 +2341,9 @@ static void canvas_paste(t_canvas *x)
 
 static void canvas_duplicate(t_canvas *x)
 {
-    if (x->gl_editor->e_onmotion == MA_NONE)
+    if (!x->gl_editor)
+        return;
+    if (x->gl_editor->e_onmotion == MA_NONE && x->gl_editor->e_selection)
     {
         t_selection *y;
         canvas_copy(x);
@@ -2168,15 +2360,55 @@ static void canvas_duplicate(t_canvas *x)
 static void canvas_selectall(t_canvas *x)
 {
     t_gobj *y;
+    if (!x->gl_editor)
+        return;
     if (!x->gl_edit)
         canvas_editmode(x, 1);
-    for (y = x->gl_list; y; y = y->g_next)
+            /* if everyone is already selected deselect everyone */
+    if (!glist_selectionindex(x, 0, 0))
+        glist_noselect(x);
+    else for (y = x->gl_list; y; y = y->g_next)
     {
         if (!glist_isselected(x, y))
             glist_select(x, y);
     }
 }
 
+static void canvas_reselect(t_canvas *x)
+{
+    t_gobj *g, *gwas;
+    t_selection *sel;
+    t_object *ob;
+        /* if someone is text editing, and if only one object is 
+        selected,  deselect everyone and reselect.  */
+    if (x->gl_editor->e_textedfor)
+    {
+            /* only do this if exactly one item is selected. */
+        if ((gwas = x->gl_editor->e_selection->sel_what) &&
+            !x->gl_editor->e_selection->sel_next)
+        {
+            int nobjwas = glist_getindex(x, 0),
+                indx = canvas_getindex(x, x->gl_editor->e_selection->sel_what);
+            glist_noselect(x);
+            for (g = x->gl_list; g; g = g->g_next)
+                if (g == gwas)
+            {
+                glist_select(x, g);
+                return;
+            }
+                /* "gwas" must have disappeared; just search to the last
+                object and select it */
+            for (g = x->gl_list; g; g = g->g_next)
+                if (!g->g_next)
+                    glist_select(x, g);
+        }
+    }
+    else if (x->gl_editor->e_selection &&
+        !x->gl_editor->e_selection->sel_next)
+            /* otherwise activate first item in selection */
+            gobj_activate(x->gl_editor->e_selection->sel_what, x, 1);
+}
+
 extern t_class *text_class;
 
 void canvas_connect(t_canvas *x, t_floatarg fwhoout, t_floatarg foutno,
@@ -2202,15 +2434,15 @@ void canvas_connect(t_canvas *x, t_floatarg fwhoout, t_floatarg foutno,
         as needed */ 
     if (pd_class(&src->g_pd) == text_class && objsrc->te_type == T_OBJECT)
         while (outno >= obj_noutlets(objsrc))
-            outlet_new(objsrc, &s_);
+            outlet_new(objsrc, 0);
     if (pd_class(&sink->g_pd) == text_class && objsink->te_type == T_OBJECT)
         while (inno >= obj_ninlets(objsink))
-            inlet_new(objsink, &objsink->ob_pd, &s_, &s_);
+            inlet_new(objsink, &objsink->ob_pd, 0, 0);
 
     if (!(oc = obj_connect(objsrc, outno, objsink, inno))) goto bad;
     if (glist_isvisible(x))
     {
-        sys_vgui(".x%lx.c create line %d %d %d %d -width %d -tags l%lx\n",
+        sys_vgui(".x%lx.c create line %d %d %d %d -width %d -tags [list l%lx cord]\n",
             glist_getcanvas(x), 0, 0, 0, 0,
             (obj_issignaloutlet(objsrc, outno) ? 2 : 1),oc);
         canvas_fixlinesfor(x, objsrc);
@@ -2224,9 +2456,9 @@ bad:
             (sink? class_getname(pd_class(&sink->g_pd)) : "???"));
 }
 
-#define XTOLERANCE 4
-#define YTOLERANCE 3
-#define NHIST 15
+#define XTOLERANCE 18
+#define YTOLERANCE 17
+#define NHIST 35
 
     /* LATER might have to speed this up */
 static void canvas_tidy(t_canvas *x)
@@ -2284,10 +2516,10 @@ static void canvas_tidy(t_canvas *x)
             }
         }
     }
-    for (i = 1, besthist = 0, bestdist = 4, ip = histogram + 1;
-        i < (NHIST-1); i++, ip++)
+    for (i = 2, besthist = 0, bestdist = 4, ip = histogram + 2;
+        i < (NHIST-2); i++, ip++)
     {
-        int hit = ip[-1] + 2 * ip[0] + ip[1];
+        int hit = ip[-2] + 2 * ip[-1] + 3 * ip[0] + 2* ip[1] + ip[2];
         if (hit > besthist)
         {
             besthist = hit;
@@ -2350,12 +2582,9 @@ void glob_key(void *dummy, t_symbol *s, int ac, t_atom *av)
     canvas_key(canvas_editing, s, ac, av);
 }
 
-void canvas_editmode(t_canvas *x, t_floatarg fyesplease)
+void canvas_editmode(t_canvas *x, t_floatarg state)
 {
-    int yesplease = fyesplease;
-    if (yesplease && x->gl_edit)
-        return;
-    x->gl_edit = !x->gl_edit;
+    x->gl_edit = (unsigned int) state;
     if (x->gl_edit && glist_isvisible(x) && glist_istoplevel(x))
         canvas_setcursor(x, CURSOR_EDITMODE_NOTHING);
     else
@@ -2364,7 +2593,7 @@ void canvas_editmode(t_canvas *x, t_floatarg fyesplease)
         if (glist_isvisible(x) && glist_istoplevel(x))
             canvas_setcursor(x, CURSOR_RUNMODE_NOTHING);
     }
-    sys_vgui("pdtk_canvas_editval .x%lx %d\n",
+    sys_vgui("pdtk_canvas_editmode .x%lx %d\n",
         glist_getcanvas(x), x->gl_edit);
 }
 
@@ -2435,7 +2664,7 @@ static void glist_setlastxy(t_glist *gl, int xval, int yval)
 void g_editor_setup(void)
 {
 /* ------------------------ events ---------------------------------- */
-    class_addmethod(canvas_class, (t_method)canvas_mousedown, gensym("mouse"),
+    class_addmethod(canvas_class, (t_method)canvas_mouse, gensym("mouse"),
         A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_NULL);
     class_addmethod(canvas_class, (t_method)canvas_mouseup, gensym("mouseup"),
         A_FLOAT, A_FLOAT, A_FLOAT, A_NULL);
@@ -2457,6 +2686,8 @@ void g_editor_setup(void)
         gensym("duplicate"), A_NULL);
     class_addmethod(canvas_class, (t_method)canvas_selectall,
         gensym("selectall"), A_NULL);
+    class_addmethod(canvas_class, (t_method)canvas_reselect,
+        gensym("reselect"), A_NULL);
     class_addmethod(canvas_class, (t_method)canvas_undo,
         gensym("undo"), A_NULL);
     class_addmethod(canvas_class, (t_method)canvas_redo,
@@ -2474,7 +2705,7 @@ void g_editor_setup(void)
     class_addmethod(canvas_class, (t_method)canvas_font,
         gensym("font"), A_FLOAT, A_FLOAT, A_FLOAT, A_NULL);
     class_addmethod(canvas_class, (t_method)canvas_find,
-        gensym("find"), A_GIMME, A_NULL);
+        gensym("find"), A_SYMBOL, A_FLOAT, A_NULL);
     class_addmethod(canvas_class, (t_method)canvas_find_again,
         gensym("findagain"), A_NULL);
     class_addmethod(canvas_class, (t_method)canvas_find_parent,
