@@ -8,6 +8,8 @@
 t_class *glob_pdobject;
 static t_class *maxclass;
 
+int sys_perf;   /* true if we should query user on close and quit */
+
 /* These "glob" routines, which implement messages to Pd, are from all
 over.  Some others are prototyped in m_imp.h as well. */
 
@@ -18,6 +20,7 @@ void glob_meters(void *dummy, t_floatarg f);
 void glob_key(void *dummy, t_symbol *s, int ac, t_atom *av);
 void glob_audiostatus(void *dummy);
 void glob_finderror(t_pd *dummy);
+void glob_findinstance(t_pd *dummy, t_symbol*s);
 void glob_audio_properties(t_pd *dummy, t_floatarg flongform);
 void glob_audio_dialog(t_pd *dummy, t_symbol *s, int argc, t_atom *argv);
 void glob_audio_setapi(t_pd *dummy, t_floatarg f);
@@ -55,6 +58,11 @@ static void glob_version(t_pd *dummy, float f)
         error("file format newer than this version of Pd (trying anyway...)");
 }
 
+static void glob_perf(t_pd *dummy, float f)
+{
+    sys_perf = (f != 0);
+}
+
 void max_default(t_pd *x, t_symbol *s, int argc, t_atom *argv)
 {
     int i;
@@ -96,6 +104,8 @@ void glob_init(void)
         gensym("audiostatus"), 0);
     class_addmethod(glob_pdobject, (t_method)glob_finderror,
         gensym("finderror"), 0);
+    class_addmethod(glob_pdobject, (t_method)glob_findinstance,
+        gensym("findinstance"), A_SYMBOL, 0);
     class_addmethod(glob_pdobject, (t_method)glob_audio_properties,
         gensym("audio-properties"), A_DEFFLOAT, 0);
     class_addmethod(glob_pdobject, (t_method)glob_audio_dialog,
@@ -121,10 +131,25 @@ void glob_init(void)
         gensym("save-preferences"), 0);
     class_addmethod(glob_pdobject, (t_method)glob_version,
         gensym("version"), A_FLOAT, 0);
-#ifdef UNIX
+    class_addmethod(glob_pdobject, (t_method)glob_perf,
+        gensym("perf"), A_FLOAT, 0);
+#if defined(__linux__) || defined(IRIX) || defined(__FreeBSD_kernel__)
     class_addmethod(glob_pdobject, (t_method)glob_watchdog,
         gensym("watchdog"), 0);
 #endif
     class_addanything(glob_pdobject, max_default);
     pd_bind(&glob_pdobject, gensym("pd"));
 }
+
+    /* function to return version number at run time.  Any of the
+    calling pointers may be zero in case you don't need all of them. */
+void sys_getversion(int *major, int *minor, int *bugfix)
+{
+    if (major)
+        *major = PD_MAJOR_VERSION;
+    if (minor)
+        *minor = PD_MINOR_VERSION;
+    if (bugfix)
+        *bugfix = PD_BUGFIX_VERSION;
+}
+
