@@ -45,7 +45,7 @@ struct _clock
 
 t_clock *clock_setlist;
 
-#ifdef UNISTD
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
@@ -272,7 +272,7 @@ static void sched_pollformeters( void)
 
         /* if there's no GUI but we're running in "realtime", here is
         where we arrange to ping the watchdog every 2 seconds. */
-#ifdef __linux__
+#if defined(__linux__) || defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__GNU__)
     if (sys_nogui && sys_hipriority && (sched_diddsp - sched_nextpingtime > 0))
     {
         glob_watchdog(0);
@@ -439,7 +439,7 @@ static void m_pollingscheduler( void)
             timeforward = sys_send_dacs();
 #ifdef THREAD_LOCKING
             /* T.Grill - done */
-            sys_unlock();
+            sys_lock();
 #endif
                 /* if dacs remain "idle" for 1 sec, they're hung up. */
             if (timeforward != 0)
@@ -450,18 +450,18 @@ static void m_pollingscheduler( void)
                 if (!(idlecount & 31))
                 {
                     static double idletime;
-                                        if (sched_useaudio != SCHED_AUDIO_POLL)
-                                        {
-                                                bug("m_pollingscheduler\n");
-                                                return;
-                                        }
+                    if (sched_useaudio != SCHED_AUDIO_POLL)
+                    {
+                            bug("m_pollingscheduler\n");
+                            return;
+                    }
                         /* on 32nd idle, start a clock watch;  every
                         32 ensuing idles, check it */
                     if (idlecount == 32)
                         idletime = sys_getrealtime();
                     else if (sys_getrealtime() - idletime > 1.)
                     {
-                        post("audio I/O stuck... closing audio\n");
+                        error("audio I/O stuck... closing audio\n");
                         sys_close_audio();
                         sched_set_using_audio(SCHED_AUDIO_NONE);
                         goto waitfortick;
@@ -522,6 +522,7 @@ static void m_pollingscheduler( void)
 
 void sched_audio_callbackfn(void)
 {
+    sys_lock();
     sys_setmiditimediff(0, 1e-6 * sys_schedadvance);
     sys_addhist(1);
     sched_tick(sys_time + sys_time_per_dsp_tick);
@@ -532,6 +533,7 @@ void sched_audio_callbackfn(void)
     sys_addhist(5);
     sched_pollformeters();
     sys_addhist(0);
+    sys_unlock();
 }
 
 static void m_callbackscheduler(void)
@@ -539,11 +541,19 @@ static void m_callbackscheduler(void)
     sys_initmidiqueue();
     while (!sys_quit)
     {
+        double timewas = sys_time;
 #ifdef MSW
-    Sleep(1000);
+        Sleep(1000);
 #else
         sleep(1);
 #endif
+        if (sys_time == timewas)
+        {
+            sys_lock();
+            sys_pollgui();
+            sched_tick(sys_time + sys_time_per_dsp_tick);
+            sys_unlock();
+        }
         if (sys_idlehook)
             sys_idlehook();
     }
@@ -566,6 +576,15 @@ int m_mainloop(void)
     return (0);
 }
 
+int m_batchmain(void)
+{
+    sys_time_per_dsp_tick = (TIMEUNITPERSEC) *
+        ((double)sys_schedblocksize) / sys_dacsr;
+    while (sys_quit != SYS_QUIT_QUIT)
+        sched_tick(sys_time + sys_time_per_dsp_tick);
+    return (0);
+}
+
 /* ------------ thread locking ------------------- */
 
 #ifdef THREAD_LOCKING
