@@ -9,17 +9,18 @@
 #include "m_pd.h"
 #include "s_stuff.h"
 #include <stdio.h>
-#ifdef UNISTD
+#ifdef _WIN32
+#include <time.h>
+#else
 #include <unistd.h>
 #include <sys/time.h>
 #include <sys/resource.h>
-#endif
+#endif /* _WIN32 */
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 
 #define SYS_DEFAULTCH 2
-#define SYS_MAXCH 100
 typedef long t_pa_sample;
 #define SYS_SAMPLEWIDTH sizeof(t_pa_sample)
 #define SYS_BYTESPERCHAN (DEFDACBLKSIZE * SYS_SAMPLEWIDTH) 
@@ -29,7 +30,7 @@ typedef long t_pa_sample;
 #define DEVDESCSIZE 80
 
 static void audio_getdevs(char *indevlist, int *nindevs,
-    char *outdevlist, int *noutdevs, int *canmulti, 
+    char *outdevlist, int *noutdevs, int *canmulti, int *cancallback, 
         int maxndev, int devdescsize);
 
     /* these are set in this file when opening audio, but then may be reduced,
@@ -37,16 +38,15 @@ static void audio_getdevs(char *indevlist, int *nindevs,
 int sys_inchannels;
 int sys_outchannels;
 int sys_advance_samples;        /* scheduler advance in samples */
-int sys_blocksize = 0;          /* audio I/O block size in sample frames */
 int sys_audioapi = API_DEFAULT;
-
+int sys_audioapiopened = -1;    /* save last API opened for later closing */
 static int sys_meters;          /* true if we're metering */
-static float sys_inmax;         /* max input amplitude */
-static float sys_outmax;        /* max output amplitude */
+static t_sample sys_inmax;         /* max input amplitude */
+static t_sample sys_outmax;        /* max output amplitude */
 
     /* exported variables */
 int sys_schedadvance;   /* scheduler advance in microseconds */
-float sys_dacsr;
+t_float sys_dacsr;
 
 t_sample *sys_soundout;
 t_sample *sys_soundin;
@@ -64,7 +64,14 @@ static int audio_naudiooutdev = -1;
 static int audio_audiooutdev[MAXAUDIOOUTDEV];
 static int audio_audiochoutdev[MAXAUDIOOUTDEV];
 static int audio_rate;
-static int audio_advance;
+static int audio_advance = -1;
+static int audio_callback;
+static int audio_blocksize;
+
+static int audio_callback_is_open;  /* reflects true actual state */
+static int audio_nextinchans, audio_nextoutchans;
+void sched_audio_callbackfn(void);
+void sched_reopenmeplease(void);
 
 static int audio_isopen(void)
 {
@@ -76,7 +83,7 @@ static int audio_isopen(void)
 void sys_get_audio_params(
     int *pnaudioindev, int *paudioindev, int *chindev,
     int *pnaudiooutdev, int *paudiooutdev, int *choutdev,
-    int *prate, int *padvance)
+    int *prate, int *padvance, int *pcallback, int *pblocksize)
 {
     int i;
     *pnaudioindev = audio_naudioindev;
@@ -89,12 +96,14 @@ void sys_get_audio_params(
             choutdev[i] = audio_audiochoutdev[i]; 
     *prate = audio_rate;
     *padvance = audio_advance;
+    *pcallback = audio_callback;
+    *pblocksize = audio_blocksize;
 }
 
 void sys_save_audio_params(
     int naudioindev, int *audioindev, int *chindev,
     int naudiooutdev, int *audiooutdev, int *choutdev,
-    int rate, int advance)
+    int rate, int advance, int callback, int blocksize)
 {
     int i;
     audio_naudioindev = naudioindev;
@@ -107,6 +116,8 @@ void sys_save_audio_params(
             audio_audiochoutdev[i] = choutdev[i]; 
     audio_rate = rate;
     audio_advance = advance;
+    audio_callback = callback;
+    audio_blocksize = blocksize;
 }
 
     /* init routines for any API which needs to set stuff up before
@@ -132,29 +143,29 @@ void sys_setchsr(int chin, int chout, int sr)
 {
     int nblk;
     int inbytes = (chin ? chin : 2) *
-                (DEFDACBLKSIZE*sizeof(float));
+                (DEFDACBLKSIZE*sizeof(t_sample));
     int outbytes = (chout ? chout : 2) *
-                (DEFDACBLKSIZE*sizeof(float));
+                (DEFDACBLKSIZE*sizeof(t_sample));
 
     if (sys_soundin)
         freebytes(sys_soundin, 
             (sys_inchannels? sys_inchannels : 2) *
-                (DEFDACBLKSIZE*sizeof(float)));
+                (DEFDACBLKSIZE*sizeof(t_sample)));
     if (sys_soundout)
         freebytes(sys_soundout, 
             (sys_outchannels? sys_outchannels : 2) *
-                (DEFDACBLKSIZE*sizeof(float)));
+                (DEFDACBLKSIZE*sizeof(t_sample)));
     sys_inchannels = chin;
     sys_outchannels = chout;
     sys_dacsr = sr;
     sys_advance_samples = (sys_schedadvance * sys_dacsr) / (1000000.);
-    if (sys_advance_samples < 3 * DEFDACBLKSIZE)
-        sys_advance_samples = 3 * DEFDACBLKSIZE;
+    if (sys_advance_samples < DEFDACBLKSIZE)
+        sys_advance_samples = DEFDACBLKSIZE;
 
-    sys_soundin = (t_float *)getbytes(inbytes);
+    sys_soundin = (t_sample *)getbytes(inbytes);
     memset(sys_soundin, 0, inbytes);
 
-    sys_soundout = (t_float *)getbytes(outbytes);
+    sys_soundout = (t_sample *)getbytes(outbytes);
     memset(sys_soundout, 0, outbytes);
 
     if (sys_verbose)
@@ -165,14 +176,14 @@ void sys_setchsr(int chin, int chout, int sr)
 
 /* ----------------------- public routines ----------------------- */
 
-    /* open audio devices (after cleaning up the specified device and channel
-    vectors).  The audio devices are "zero based" (i.e. "0" means the first
-    one.)  We also save the cleaned-up device specification so that we
-    can later re-open audio and/or show the settings on a dialog window. */
+    /* set audio device settings (after cleaning up the specified device and
+    channel vectors).  The audio devices are "zero based" (i.e. "0" means the
+    first one.)  We can later re-open audio and/or show the settings on a
+    dialog window. */
 
-void sys_open_audio(int naudioindev, int *audioindev, int nchindev,
+void sys_set_audio_settings(int naudioindev, int *audioindev, int nchindev,
     int *chindev, int naudiooutdev, int *audiooutdev, int nchoutdev,
-    int *choutdev, int rate, int advance, int enable)
+    int *choutdev, int rate, int advance, int callback, int blocksize)
 {
     int i, *ip;
     int defaultchannels = SYS_DEFAULTCH;
@@ -181,22 +192,16 @@ void sys_open_audio(int naudioindev, int *audioindev, int nchindev,
     int realinchans[MAXAUDIOINDEV], realoutchans[MAXAUDIOOUTDEV];
 
     char indevlist[MAXNDEV*DEVDESCSIZE], outdevlist[MAXNDEV*DEVDESCSIZE];
-    int indevs = 0, outdevs = 0, canmulti = 0;
+    int indevs = 0, outdevs = 0, canmulti = 0, cancallback = 0;
     audio_getdevs(indevlist, &indevs, outdevlist, &outdevs, &canmulti,
-        MAXNDEV, DEVDESCSIZE);
-
-    if (sys_externalschedlib)
-    {
-        return;
-    }
-        /* if we're already open close it */
-    if (sys_inchannels || sys_outchannels)
-        sys_close_audio();
+        &cancallback, MAXNDEV, DEVDESCSIZE);
 
     if (rate < 1)
         rate = DEFAULTSRATE;
-    if (advance <= 0)
+    if (advance < 0)
         advance = DEFAULTADVANCE;
+    if (blocksize != (1 << ilog2(blocksize)) || blocksize < DEFDACBLKSIZE)
+        blocksize = DEFDACBLKSIZE;
      audio_init();
         /* Since the channel vector might be longer than the
         audio device vector, or vice versa, we fill the shorter one
@@ -319,73 +324,13 @@ void sys_open_audio(int naudioindev, int *audioindev, int nchindev,
         outchans += choutdev[i];
         nrealoutdev++;
     }
-        /* if no input or output devices seem to have been specified,
-        this really means just disable audio, which we now do. */
-    if (!inchans && !outchans)
-        enable = 0;
     sys_schedadvance = advance * 1000;
-    sys_setchsr(inchans, outchans, rate);
     sys_log_error(ERR_NOTHING);
-    if (enable)
-    {
-#ifdef USEAPI_PORTAUDIO
-        if (sys_audioapi == API_PORTAUDIO)
-        {
-            int blksize = (sys_blocksize ? sys_blocksize : 64);
-            pa_open_audio(inchans, outchans, rate, sys_soundin, sys_soundout,
-                blksize, sys_advance_samples/blksize,
-                    (naudiooutdev > 0 ? audioindev[0] : 0),
-                        (naudiooutdev > 0 ? audiooutdev[0] : 0));
-        }
-else
-#endif
-#ifdef USEAPI_JACK
-        if (sys_audioapi == API_JACK) 
-            jack_open_audio((nrealindev > 0 ? realinchans[0] : 0),
-                (nrealoutdev > 0 ? realoutchans[0] : 0), rate);
-
-        else
-#endif    
-#ifdef USEAPI_OSS
-        if (sys_audioapi == API_OSS)
-            oss_open_audio(nrealindev, realindev, nrealindev, realinchans,
-                nrealoutdev, realoutdev, nrealoutdev, realoutchans, rate);
-        else
-#endif
-#ifdef USEAPI_ALSA
-            /* for alsa, only one device is supported; it may
-            be open for both input and output. */
-        if (sys_audioapi == API_ALSA)
-            alsa_open_audio(nrealindev, audioindev, nrealindev, realinchans,
-                nrealoutdev, audiooutdev, nrealoutdev, realoutchans, rate);
-        else 
-#endif
-#ifdef USEAPI_SGI
-        if (sys_audioapi == API_SGI)
-        {
-            xtern int sgi_open_audio(int nindev,  int *indev,  int nchin,
-                int *chin,  int noutdev, int *outdev, int nchout, int *chout,
-                int rate);
-            sgi_open_audio(naudioindev, audioindev, nchindev, chindev,
-                    naudiooutdev, audiooutdev, nchoutdev, choutdev, rate);
-        }
-        else
-#endif
-#ifdef USEAPI_MMIO
-        if (sys_audioapi == API_MMIO)
-            mmio_open_audio(nrealindev, audioindev, nrealindev, realinchans,
-                nrealoutdev, audiooutdev, nrealoutdev, realoutchans, rate);
-        else
-#endif
-            post("unknown audio API specified");
-    }
-    sys_save_audio_params(naudioindev, audioindev, chindev,
-        naudiooutdev, audiooutdev, choutdev, sys_dacsr, advance);
-    if (sys_inchannels == 0 && sys_outchannels == 0)
-        enable = 0;
-    audio_state = enable;
-    sys_vgui("set pd_whichapi %d\n",  (audio_isopen() ? sys_audioapi : 0));
-    sched_set_using_dacs(enable);
+    audio_nextinchans = inchans;
+    audio_nextoutchans = outchans;
+    sys_save_audio_params(nrealindev, realindev, realinchans,
+        nrealoutdev, realoutdev, realoutchans, rate, advance, callback,
+            blocksize);
 }
 
 void sys_close_audio(void)
@@ -397,40 +342,53 @@ void sys_close_audio(void)
     if (!audio_isopen())
         return;
 #ifdef USEAPI_PORTAUDIO
-    if (sys_audioapi == API_PORTAUDIO)
+    if (sys_audioapiopened == API_PORTAUDIO)
         pa_close_audio();
     else 
 #endif
 #ifdef USEAPI_JACK
-    if (sys_audioapi == API_JACK)
+    if (sys_audioapiopened == API_JACK)
         jack_close_audio();
     else
 #endif
 #ifdef USEAPI_OSS
-    if (sys_audioapi == API_OSS)
+    if (sys_audioapiopened == API_OSS)
         oss_close_audio();
     else
 #endif
 #ifdef USEAPI_ALSA
-    if (sys_audioapi == API_ALSA)
+    if (sys_audioapiopened == API_ALSA)
         alsa_close_audio();
     else
 #endif
-#ifdef USEAPI_SGI
-    if (sys_audioapi == API_SGI)
-    {
-        extern void sgi_close_audio(void);
-        sgi_close_audio();
-    }
-    else
-#endif
 #ifdef USEAPI_MMIO
-    if (sys_audioapi == API_MMIO)
+    if (sys_audioapiopened == API_MMIO)
         mmio_close_audio();
     else
 #endif
-        post("sys_close_audio: unknown API %d", sys_audioapi);
+#ifdef USEAPI_AUDIOUNIT
+    if (sys_audioapiopened == API_AUDIOUNIT)
+        audiounit_close_audio();
+    else
+#endif
+#ifdef USEAPI_ESD
+    if (sys_audioapiopened == API_ESD)
+        esd_close_audio();
+    else
+#endif
+#ifdef USEAPI_DUMMY
+    if (sys_audioapiopened == API_DUMMY)
+        dummy_close_audio();
+    else
+#endif
+        post("sys_close_audio: unknown API %d", sys_audioapiopened);
     sys_inchannels = sys_outchannels = 0;
+    sys_audioapiopened = -1;
+    sched_set_using_audio(SCHED_AUDIO_NONE);
+    audio_state = 0;
+    audio_callback_is_open = 0;
+
+    sys_vgui("set pd_whichapi 0\n");
 }
 
     /* open audio using whatever parameters were last used */
@@ -438,11 +396,99 @@ void sys_reopen_audio( void)
 {
     int naudioindev, audioindev[MAXAUDIOINDEV], chindev[MAXAUDIOINDEV];
     int naudiooutdev, audiooutdev[MAXAUDIOOUTDEV], choutdev[MAXAUDIOOUTDEV];
-    int rate, advance;
+    int rate, advance, callback, blocksize, outcome = 0;
     sys_get_audio_params(&naudioindev, audioindev, chindev,
-        &naudiooutdev, audiooutdev, choutdev, &rate, &advance);
-    sys_open_audio(naudioindev, audioindev, naudioindev, chindev,
-        naudiooutdev, audiooutdev, naudiooutdev, choutdev, rate, advance, 1);
+        &naudiooutdev, audiooutdev, choutdev, &rate, &advance, &callback,
+            &blocksize);
+    sys_setchsr(audio_nextinchans, audio_nextoutchans, rate);
+    if (!naudioindev && !naudiooutdev)
+    {
+        sched_set_using_audio(SCHED_AUDIO_NONE);
+        return;
+    }
+#ifdef USEAPI_PORTAUDIO
+    if (sys_audioapi == API_PORTAUDIO)
+    {
+        int blksize = (audio_blocksize ? audio_blocksize : 64);
+        if (sys_verbose)
+            fprintf(stderr, "blksize %d, advance %d\n", blksize, sys_advance_samples/blksize);
+        outcome = pa_open_audio((naudioindev > 0 ? chindev[0] : 0),
+        (naudiooutdev > 0 ? choutdev[0] : 0), rate, sys_soundin,
+            sys_soundout, blksize, sys_advance_samples/blksize, 
+             (naudioindev > 0 ? audioindev[0] : 0),
+              (naudiooutdev > 0 ? audiooutdev[0] : 0),
+               (callback ? sched_audio_callbackfn : 0));
+    }
+    else
+#endif
+#ifdef USEAPI_JACK
+    if (sys_audioapi == API_JACK) 
+        outcome = jack_open_audio((naudioindev > 0 ? chindev[0] : 0),
+            (naudioindev > 0 ? choutdev[0] : 0), rate,
+                (callback ? sched_audio_callbackfn : 0));
+
+    else
+#endif    
+#ifdef USEAPI_OSS
+    if (sys_audioapi == API_OSS)
+        outcome = oss_open_audio(naudioindev, audioindev, naudioindev,
+            chindev, naudiooutdev, audiooutdev, naudiooutdev, choutdev, rate,
+                audio_blocksize);
+    else
+#endif
+#ifdef USEAPI_ALSA
+        /* for alsa, only one device is supported; it may
+        be open for both input and output. */
+    if (sys_audioapi == API_ALSA)
+        outcome = alsa_open_audio(naudioindev, audioindev, naudioindev,
+            chindev, naudiooutdev, audiooutdev, naudiooutdev, choutdev, rate,
+                audio_blocksize);
+    else 
+#endif
+#ifdef USEAPI_MMIO
+    if (sys_audioapi == API_MMIO)
+        outcome = mmio_open_audio(naudioindev, audioindev, naudioindev,
+            chindev, naudiooutdev, audiooutdev, naudiooutdev, choutdev, rate,
+                audio_blocksize);
+    else
+#endif
+#ifdef USEAPI_AUDIOUNIT
+    if (sys_audioapi == API_AUDIOUNIT)
+        outcome = audiounit_open_audio((naudioindev > 0 ? chindev[0] : 0),
+            (naudioindev > 0 ? choutdev[0] : 0), rate);
+    else
+#endif
+#ifdef USEAPI_ESD
+    if (sys_audioapi == API_ALSA)
+        outcome = esd_open_audio(naudioindev, audioindev, naudioindev,
+            chindev, naudiooutdev, audiooutdev, naudiooutdev, choutdev, rate);
+    else 
+#endif
+#ifdef USEAPI_DUMMY
+    if (sys_audioapi == API_DUMMY)
+        outcome = dummy_open_audio(naudioindev, naudiooutdev, rate);
+    else
+#endif
+    if (sys_audioapi == API_NONE)
+        ;
+    else post("unknown audio API specified");
+    if (outcome)    /* failed */
+    {
+        audio_state = 0;
+        sched_set_using_audio(SCHED_AUDIO_NONE);
+        sys_audioapiopened = -1;
+        audio_callback_is_open = 0;
+    }
+    else
+    {
+                /* fprintf(stderr, "started w/callback %d\n", callback); */
+        audio_state = 1;
+        sched_set_using_audio(
+            (callback ? SCHED_AUDIO_CALLBACK : SCHED_AUDIO_POLL));
+        sys_audioapiopened = sys_audioapi;
+        audio_callback_is_open = callback;
+    }
+    sys_vgui("set pd_whichapi %d\n",  (outcome == 0 ? sys_audioapi : 0));
 }
 
 int sys_send_dacs(void)
@@ -450,11 +496,11 @@ int sys_send_dacs(void)
     if (sys_meters)
     {
         int i, n;
-        float maxsamp;
+        t_sample maxsamp;
         for (i = 0, n = sys_inchannels * DEFDACBLKSIZE, maxsamp = sys_inmax;
             i < n; i++)
         {
-            float f = sys_soundin[i];
+            t_sample f = sys_soundin[i];
             if (f > maxsamp) maxsamp = f;
             else if (-f > maxsamp) maxsamp = -f;
         }
@@ -462,7 +508,7 @@ int sys_send_dacs(void)
         for (i = 0, n = sys_outchannels * DEFDACBLKSIZE, maxsamp = sys_outmax;
             i < n; i++)
         {
-            float f = sys_soundout[i];
+            t_sample f = sys_soundout[i];
             if (f > maxsamp) maxsamp = f;
             else if (-f > maxsamp) maxsamp = -f;
         }
@@ -489,24 +535,31 @@ int sys_send_dacs(void)
         return (alsa_send_dacs());
     else
 #endif
-#ifdef USEAPI_SGI
-    if (sys_audioapi == API_SGI)
-    {
-        extern int sgi_send_dacs(void);
-        return (sgi_send_dacs());
-    }
-    else
-#endif
 #ifdef USEAPI_MMIO
     if (sys_audioapi == API_MMIO)
         return (mmio_send_dacs());
     else
+#endif
+#ifdef USEAPI_AUDIOUNIT
+    if (sys_audioapi == API_AUDIOUNIT)
+        return (audiounit_send_dacs());
+    else
+#endif
+#ifdef USEAPI_ESD
+    if (sys_audioapi == API_ESD)
+        return (esd_send_dacs());
+    else
+#endif
+#ifdef USEAPI_DUMMY
+    if (sys_audioapi == API_DUMMY)
+        return (dummy_send_dacs());
+    else
 #endif
     post("unknown API");    
     return (0);
 }
 
-float sys_getsr(void)
+t_float sys_getsr(void)
 {
      return (sys_dacsr);
 }
@@ -521,7 +574,7 @@ int sys_get_inchannels(void)
      return (sys_inchannels);
 }
 
-void sys_getmeters(float *inmax, float *outmax)
+void sys_getmeters(t_sample *inmax, t_sample *outmax)
 {
     if (inmax)
     {
@@ -539,15 +592,17 @@ void sys_reportidle(void)
 }
 
 static void audio_getdevs(char *indevlist, int *nindevs,
-    char *outdevlist, int *noutdevs, int *canmulti, 
+    char *outdevlist, int *noutdevs, int *canmulti, int *cancallback,
         int maxndev, int devdescsize)
 {
     audio_init();
+    *cancallback = 0;   /* may be overridden by specific API implementation */
 #ifdef USEAPI_PORTAUDIO
     if (sys_audioapi == API_PORTAUDIO)
     {
         pa_getdevs(indevlist, nindevs, outdevlist, noutdevs, canmulti,
             maxndev, devdescsize);
+        *cancallback = 1;
     }
     else
 #endif
@@ -556,6 +611,7 @@ static void audio_getdevs(char *indevlist, int *nindevs,
     {
         jack_getdevs(indevlist, nindevs, outdevlist, noutdevs, canmulti,
             maxndev, devdescsize);
+        *cancallback = 1;
     }
     else
 #endif
@@ -575,17 +631,6 @@ static void audio_getdevs(char *indevlist, int *nindevs,
     }
     else
 #endif
-#ifdef USEAPI_SGI
-    if (sys_audioapi == API_SGI)
-    {
-        extern void sgi_getdevs(char *indevlist, int *nindevs,
-            char *outdevlist, int *noutdevs, int *canmulti, 
-            int maxndev, int devdescsize);
-        sgi_getdevs(indevlist, nindevs, outdevlist, noutdevs, canmulti,
-            maxndev, devdescsize);
-    }
-    else
-#endif
 #ifdef USEAPI_MMIO
     if (sys_audioapi == API_MMIO)
     {
@@ -593,6 +638,28 @@ static void audio_getdevs(char *indevlist, int *nindevs,
             maxndev, devdescsize);
     }
     else
+#endif
+#ifdef USEAPI_AUDIOUNIT
+    if (sys_audioapi == API_AUDIOUNIT)
+    {
+    }
+    else
+#endif
+#ifdef USEAPI_ESD
+    if (sys_audioapi == API_ESD)
+    {
+        esd_getdevs(indevlist, nindevs, outdevlist, noutdevs, canmulti,
+            maxndev, devdescsize);
+    }
+    else
+#endif
+#ifdef USEAPI_DUMMY
+    if (sys_audioapi == API_DUMMY)
+    {
+        dummy_getdevs(indevlist, nindevs, outdevlist, noutdevs, canmulti,
+            maxndev, devdescsize);
+    }
+    else
 #endif
     {
             /* this shouldn't happen once all the above get filled in. */
@@ -611,10 +678,10 @@ static void audio_getdevs(char *indevlist, int *nindevs,
 static void sys_listaudiodevs(void )
 {
     char indevlist[MAXNDEV*DEVDESCSIZE], outdevlist[MAXNDEV*DEVDESCSIZE];
-    int nindevs = 0, noutdevs = 0, i, canmulti = 0;
+    int nindevs = 0, noutdevs = 0, i, canmulti = 0, cancallback = 0;
 
     audio_getdevs(indevlist, &nindevs, outdevlist, &noutdevs, &canmulti,
-        MAXNDEV, DEVDESCSIZE);
+        &cancallback, MAXNDEV, DEVDESCSIZE);
 
     if (!nindevs)
         post("no audio input devices found");
@@ -653,26 +720,27 @@ void glob_audio_properties(t_pd *dummy, t_floatarg flongform)
         audioinchan1, audioinchan2, audioinchan3, audioinchan4,
         audiooutdev1, audiooutdev2, audiooutdev3, audiooutdev4,
         audiooutchan1, audiooutchan2, audiooutchan3, audiooutchan4;
-    int rate, advance;
+    int rate, advance, callback, blocksize;
         /* these are all the devices on your system: */
     char indevlist[MAXNDEV*DEVDESCSIZE], outdevlist[MAXNDEV*DEVDESCSIZE];
-    int nindevs = 0, noutdevs = 0, canmulti = 0, i;
+    int nindevs = 0, noutdevs = 0, canmulti = 0, cancallback = 0, i;
 
     audio_getdevs(indevlist, &nindevs, outdevlist, &noutdevs, &canmulti,
-        MAXNDEV, DEVDESCSIZE);
+         &cancallback, MAXNDEV, DEVDESCSIZE);
 
     sys_gui("global audio_indevlist; set audio_indevlist {}\n");
     for (i = 0; i < nindevs; i++)
-        sys_vgui("lappend audio_indevlist \"%s\"\n",
+        sys_vgui("lappend audio_indevlist {%s}\n",
             indevlist + i * DEVDESCSIZE);
 
     sys_gui("global audio_outdevlist; set audio_outdevlist {}\n");
     for (i = 0; i < noutdevs; i++)
-        sys_vgui("lappend audio_outdevlist \"%s\"\n",
+        sys_vgui("lappend audio_outdevlist {%s}\n",
             outdevlist + i * DEVDESCSIZE);
 
     sys_get_audio_params(&naudioindev, audioindev, chindev,
-        &naudiooutdev, audiooutdev, choutdev, &rate, &advance);
+        &naudiooutdev, audiooutdev, choutdev, &rate, &advance, &callback,
+            &blocksize);
 
     /* post("naudioindev %d naudiooutdev %d longform %f",
             naudioindev, naudiooutdev, flongform); */
@@ -699,16 +767,18 @@ void glob_audio_properties(t_pd *dummy, t_floatarg flongform)
 "pdtk_audio_dialog %%s \
 %d %d %d %d %d %d %d %d \
 %d %d %d %d %d %d %d %d \
-%d %d %d %d\n",
+%d %d %d %d %d %d\n",
         audioindev1, audioindev2, audioindev3, audioindev4, 
         audioinchan1, audioinchan2, audioinchan3, audioinchan4, 
         audiooutdev1, audiooutdev2, audiooutdev3, audiooutdev4,
         audiooutchan1, audiooutchan2, audiooutchan3, audiooutchan4, 
-        rate, advance, canmulti, (flongform != 0));
+        rate, advance, canmulti, (cancallback ? callback : -1),
+        (flongform != 0), blocksize);
     gfxstub_deleteforkey(0);
     gfxstub_new(&glob_pdobject, (void *)glob_audio_properties, buf);
 }
 
+extern int pa_foo;
     /* new values from dialog window */
 void glob_audio_dialog(t_pd *dummy, t_symbol *s, int argc, t_atom *argv)
 {
@@ -721,7 +791,8 @@ void glob_audio_dialog(t_pd *dummy, t_symbol *s, int argc, t_atom *argv)
         /* the new values the dialog came back with: */
     int newrate = atom_getintarg(16, argc, argv);
     int newadvance = atom_getintarg(17, argc, argv);
-    int statewas;
+    int newcallback = atom_getintarg(18, argc, argv);
+    int newblocksize = atom_getintarg(19, argc, argv);
 
     for (i = 0; i < 4; i++)
     {
@@ -753,11 +824,30 @@ void glob_audio_dialog(t_pd *dummy, t_symbol *s, int argc, t_atom *argv)
             noutdev++;
         }
     }
-
-    sys_close_audio();
-    sys_open_audio(nindev, newaudioindev, nindev, newaudioinchan,
+    
+    sys_set_audio_settings_reopen(nindev, newaudioindev, nindev, newaudioinchan,
         noutdev, newaudiooutdev, noutdev, newaudiooutchan,
-        newrate, newadvance, 1);
+        newrate, newadvance, newcallback, newblocksize);
+}
+
+void sys_set_audio_settings_reopen(int naudioindev, int *audioindev, int nchindev,
+    int *chindev, int naudiooutdev, int *audiooutdev, int nchoutdev,
+    int *choutdev, int rate, int advance, int callback, int newblocksize)
+{
+    if (callback < 0)
+        callback = 0;
+    if (newblocksize != (1<<ilog2(newblocksize)) ||
+        newblocksize < DEFDACBLKSIZE || newblocksize > 2048)
+            newblocksize = DEFDACBLKSIZE;
+    
+    if (!audio_callback_is_open && !callback)
+        sys_close_audio();
+    sys_set_audio_settings(naudioindev, audioindev, nchindev, chindev,
+        naudiooutdev, audiooutdev, nchoutdev, choutdev,
+        rate, advance, (callback >= 0 ? callback : 0), newblocksize);
+    if (!audio_callback_is_open && !callback)
+        sys_reopen_audio();
+    else sched_reopenmeplease();
 }
 
 void sys_listdevs(void )
@@ -782,30 +872,39 @@ void sys_listdevs(void )
         sys_listaudiodevs();
     else
 #endif
-#ifdef USEAPI_SGI
-        extern void sgi_listaudiodevs(void);
-    if (sys_audioapi == API_SGI)
-        sgi_listaudiodevs();
-    else
-#endif
 #ifdef USEAPI_MMIO
     if (sys_audioapi == API_MMIO)
         sys_listaudiodevs();
     else
+#endif
+#ifdef USEAPI_AUDIOUNIT
+    if (sys_audioapi == API_AUDIOUNIT)
+        sys_listaudiodevs();
+    else
+#endif
+#ifdef USEAPI_ESD
+    if (sys_audioapi == API_ESD)
+        sys_listaudiodevs();
+    else
+#endif
+#ifdef USEAPI_DUMMY
+    if (sys_audioapi == API_DUMMY)
+        sys_listaudiodevs();
+    else
 #endif
     post("unknown API");    
 
     sys_listmididevs();
 }
 
-void sys_setblocksize(int n)
+void sys_get_audio_devs(char *indevlist, int *nindevs,
+    char *outdevlist, int *noutdevs, int *canmulti, int *cancallback, 
+                        int maxndev, int devdescsize)
 {
-    if (n < 1)
-        n = 1;
-    if (n != (1 << ilog2(n)))
-        post("warning: adjusting blocksize to power of 2: %d", 
-            (n = (1 << ilog2(n))));
-    sys_blocksize = n;
+  audio_getdevs(indevlist, nindevs,
+                outdevlist, noutdevs, 
+                canmulti, cancallback, 
+                maxndev, devdescsize);
 }
 
 void sys_set_audio_api(int which)
@@ -840,8 +939,6 @@ void glob_audio_setapi(void *dummy, t_floatarg f)
     else if (audio_isopen())
     {
         sys_close_audio();
-        audio_state = 0;
-        sched_set_using_dacs(0);
     }
 }
 
@@ -856,12 +953,8 @@ void sys_set_audio_state(int onoff)
     else
     {
         if (audio_isopen())
-        {
             sys_close_audio();
-            sched_set_using_dacs(0);
-        }
     }
-    audio_state = onoff;
 }
 
 void sys_get_audio_apis(char *buf)
@@ -888,20 +981,25 @@ void sys_get_audio_apis(char *buf)
 #else
     sprintf(buf + strlen(buf), "{portaudio %d} ", API_PORTAUDIO);
 #endif
-#ifdef USEAPI_SGI
-    sprintf(buf + strlen(buf), "{SGI %d} ", API_SGI); n++;
-#endif
 #endif
      n++;
 #endif
 #ifdef USEAPI_JACK
     sprintf(buf + strlen(buf), "{jack %d} ", API_JACK); n++;
+#endif
+#ifdef USEAPI_AUDIOUNIT
+    sprintf(buf + strlen(buf), "{AudioUnit %d} ", API_AUDIOUNIT); n++;
+#endif
+#ifdef USEAPI_ESD
+    sprintf(buf + strlen(buf), "{ESD %d} ", API_ESD); n++;
+#endif
+#ifdef USEAPI_DUMMY
+    sprintf(buf + strlen(buf), "{dummy %d} ", API_DUMMY); n++;
 #endif
     strcat(buf, "}");
         /* then again, if only one API (or none) we don't offer any choice. */
     if (n < 2)
         strcpy(buf, "{}");
-    
 }
 
 #ifdef USEAPI_ALSA
