@@ -25,6 +25,7 @@
 #include <sched.h>
 #include <sys/mman.h>
 #include "s_audio_alsa.h"
+#include <endian.h>
 
 /* Defines */
 #define DEBUG(x) x
@@ -89,9 +90,17 @@ static int alsaio_canmmap(t_alsa_dev *dev)
     return ((err1 < 0) && (err2 >= 0));
 }
 
+static void check_setup_error(int err, int out, const char *why) {
+        char bf[256];
+        snprintf(bf, sizeof bf, "%s (%s)", why, out ? "output" : "input");
+        check_error(err, bf);
+}
+
 static int alsaio_setup(t_alsa_dev *dev, int out, int *channels, int *rate,
     int nfrags, int frag_size)
 {
+#define CHECK_ERROR(why_) check_setup_error(err, out, why_)
+
     int bufsizeforthis, err;
     snd_pcm_hw_params_t* hw_params;
     unsigned int tmp_uint;
@@ -109,25 +118,41 @@ static int alsaio_setup(t_alsa_dev *dev, int out, int *channels, int *rate,
 
         /* get the default params */
     err = snd_pcm_hw_params_any(dev->a_handle, hw_params);
-    check_error(err, "snd_pcm_hw_params_any");
+    CHECK_ERROR("snd_pcm_hw_params_any");
 
         /* try to set interleaved access */
     err = snd_pcm_hw_params_set_access(dev->a_handle,
         hw_params, SND_PCM_ACCESS_RW_INTERLEAVED);
     if (err < 0)
         return (-1);
-    check_error(err, "snd_pcm_hw_params_set_access");
+    CHECK_ERROR("snd_pcm_hw_params_set_access");
+#if 0       /* enable this to print out which formats are available */
+    {
+        int i;
+        for (i = 0; i <= SND_PCM_FORMAT_LAST; i++)
+            fprintf(stderr, "%d -> %d\n",
+                i, snd_pcm_hw_params_test_format(dev->a_handle, hw_params, i));
+    }
+#endif
         /* Try to set 32 bit format first */
     err = snd_pcm_hw_params_set_format(dev->a_handle,
         hw_params, SND_PCM_FORMAT_S32);
     if (err < 0)
     {
         /* fprintf(stderr,
-            "PD-ALSA: 32 bit format not available - using 16\n"); */
+            "PD-ALSA: 32 bit format not available - trying 24\n"); */
         err = snd_pcm_hw_params_set_format(dev->a_handle, hw_params,
-                                           SND_PCM_FORMAT_S16);
-        check_error(err, "snd_pcm_hw_params_set_format");
-        dev->a_sampwidth = 2;
+            SND_PCM_FORMAT_S24_3LE);
+        if (err < 0)
+        {
+            /* fprintf(stderr,
+                "PD-ALSA: 32/24 bit format not available - using 16\n"); */
+            err = snd_pcm_hw_params_set_format(dev->a_handle, hw_params,
+                SND_PCM_FORMAT_S16);
+            CHECK_ERROR("_pcm_hw_params_set_format");
+            dev->a_sampwidth = 2;
+        }
+        else dev->a_sampwidth = 3;
     }
     else dev->a_sampwidth = 4;
 
@@ -137,31 +162,29 @@ static int alsaio_setup(t_alsa_dev *dev, int out, int *channels, int *rate,
         /* set the subformat */
     err = snd_pcm_hw_params_set_subformat(dev->a_handle,
         hw_params, SND_PCM_SUBFORMAT_STD);
-    check_error(err, "snd_pcm_hw_params_set_subformat");
+    CHECK_ERROR("snd_pcm_hw_params_set_subformat");
     
         /* set the number of channels */
     tmp_uint = *channels;
     err = snd_pcm_hw_params_set_channels_min(dev->a_handle,
         hw_params, &tmp_uint);
-    check_error(err, "snd_pcm_hw_params_set_channels");
+    CHECK_ERROR("snd_pcm_hw_params_set_channels");
     if (tmp_uint != (unsigned)*channels)
-        post("ALSA: set input channels to %d", tmp_uint);
+        post("ALSA: set %s channels to %d", (out?"output":"input"), tmp_uint);
     *channels = tmp_uint;
     dev->a_channels = *channels;
 
         /* set the sampling rate */
-    err = snd_pcm_hw_params_set_rate_min(dev->a_handle, hw_params, 
+    err = snd_pcm_hw_params_set_rate_near(dev->a_handle, hw_params, 
         (unsigned int *)rate, 0);
-    check_error(err, "snd_pcm_hw_params_set_rate_min (input)");
+    CHECK_ERROR("snd_pcm_hw_params_set_rate_min");
 #if 0
     err = snd_pcm_hw_params_get_rate(hw_params, &subunitdir);
     post("input sample rate %d", err);
 #endif
 
-        /* set the period - ie frag size */
-        /* LATER try this to get a recommended period size...
-         right now, it trips an assertion failure in ALSA lib */
-
+    /* post("frag size %d, nfrags %d", frag_size, nfrags); */
+        /* set "period size" */
 #ifdef ALSAAPI9
     err = snd_pcm_hw_params_set_period_size_near(dev->a_handle,
         hw_params, (snd_pcm_uframes_t)frag_size, 0);
@@ -170,18 +193,7 @@ static int alsaio_setup(t_alsa_dev *dev, int out, int *channels, int *rate,
     err = snd_pcm_hw_params_set_period_size_near(dev->a_handle,
         hw_params, &tmp_snd_pcm_uframes, 0);
 #endif
-    check_error(err, "snd_pcm_hw_params_set_period_size_near (input)");
-    
-        /* set the number of periods - ie numfrags */
-#ifdef ALSAAPI9
-    err = snd_pcm_hw_params_set_periods_near(dev->a_handle,
-        hw_params, nfrags, 0);
-#else
-    tmp_uint = nfrags;
-    err = snd_pcm_hw_params_set_periods_near(dev->a_handle,
-        hw_params, &tmp_uint, 0);
-#endif
-    check_error(err, "snd_pcm_hw_params_set_periods_near (input)");
+    CHECK_ERROR("snd_pcm_hw_params_set_period_size_near");
 
         /* set the buffer size */
 #ifdef ALSAAPI9
@@ -192,10 +204,10 @@ static int alsaio_setup(t_alsa_dev *dev, int out, int *channels, int *rate,
     err = snd_pcm_hw_params_set_buffer_size_near(dev->a_handle,
         hw_params, &tmp_snd_pcm_uframes);
 #endif
-    check_error(err, "snd_pcm_hw_params_set_buffer_size_near (input)");
+    CHECK_ERROR("snd_pcm_hw_params_set_buffer_size_near");
 
     err = snd_pcm_hw_params(dev->a_handle, hw_params);
-    check_error(err, "snd_pcm_hw_params (input)");
+    CHECK_ERROR("snd_pcm_hw_params");
 
         /* set up the buffer */
     bufsizeforthis = DEFDACBLKSIZE * dev->a_sampwidth * *channels;
@@ -223,18 +235,19 @@ static int alsaio_setup(t_alsa_dev *dev, int out, int *channels, int *rate,
         alsa_snd_bufsize = bufsizeforthis;
     }
     return (1);
+#undef CHECK_ERROR
 }
 
 
     /* return 0 on success */
 int alsa_open_audio(int naudioindev, int *audioindev, int nchindev,
     int *chindev, int naudiooutdev, int *audiooutdev, int nchoutdev,
-    int *choutdev, int rate)
+    int *choutdev, int rate, int blocksize)
 {
     int err, inchans = 0, outchans = 0, subunitdir;
     char devname[512];
     snd_output_t* out;
-    int frag_size = (sys_blocksize ? sys_blocksize : ALSA_DEFFRAGSIZE);
+    int frag_size = (blocksize ? blocksize : ALSA_DEFFRAGSIZE);
     int nfrags, i, iodev, dev2;
     int wantinchans, wantoutchans, device;
 
@@ -288,7 +301,7 @@ int alsa_open_audio(int naudioindev, int *audioindev, int nchindev,
     if (alsa_usemmap)
     {
         post("using mmap audio interface");
-        if (alsamm_open_audio(rate))
+        if (alsamm_open_audio(rate, blocksize))
             goto blewit;
         else return (0);
     }
@@ -429,7 +442,6 @@ int alsa_send_dacs(void)
         if (snd_pcm_status_get_avail(alsa_status) < transfersize)
             return SENDDACS_NO;
     }
-
     /* do output */
     for (iodev = 0, fp1 = sys_soundout, ch = 0; iodev < alsa_noutdev; iodev++)
     {
@@ -440,17 +452,42 @@ int alsa_send_dacs(void)
         if (alsa_outdev[iodev].a_sampwidth == 4)
         {
             for (i = 0; i < chans; i++, ch++, fp1 += DEFDACBLKSIZE)
-                for (j = ch, k = DEFDACBLKSIZE, fp2 = fp1; k--;
+                for (j = i, k = DEFDACBLKSIZE, fp2 = fp1; k--;
                      j += thisdevchans, fp2++)
             {
                 float s1 = *fp2 * INT32_MAX;
                 ((t_alsa_sample32 *)alsa_snd_buf)[j] = CLIP32(s1);
             } 
             for (; i < thisdevchans; i++, ch++)
-                for (j = ch, k = DEFDACBLKSIZE; k--; j += thisdevchans)
+                for (j = i, k = DEFDACBLKSIZE; k--; j += thisdevchans)
                     ((t_alsa_sample32 *)alsa_snd_buf)[j] = 0;
         }
-        else
+        else if (alsa_outdev[iodev].a_sampwidth == 3)
+        {
+            for (i = 0; i < chans; i++, ch++, fp1 += DEFDACBLKSIZE)
+                for (j = i, k = DEFDACBLKSIZE, fp2 = fp1; k--;
+                     j += thisdevchans, fp2++)
+            {
+                int s = *fp2 * 8388352.;
+                if (s > 8388351)
+                    s = 8388351;
+                else if (s < -8388351)
+                    s = -8388351;
+#if BYTE_ORDER == LITTLE_ENDIAN                                             
+                ((char *)(alsa_snd_buf))[3*j] = (s & 255);
+                ((char *)(alsa_snd_buf))[3*j+1] = ((s>>8) & 255);
+                ((char *)(alsa_snd_buf))[3*j+2] = ((s>>16) & 255);
+#else
+                fprintf(stderr, "big endian 24-bit not supported");
+#endif
+            }
+            for (; i < thisdevchans; i++, ch++)
+                for (j = i, k = DEFDACBLKSIZE; k--; j += thisdevchans)
+                    ((char *)(alsa_snd_buf))[3*j] = 
+                    ((char *)(alsa_snd_buf))[3*j+1] = 
+                    ((char *)(alsa_snd_buf))[3*j+2] = 0;
+        }
+        else        /* 16 bit samples */
         {
             for (i = 0; i < chans; i++, ch++, fp1 += DEFDACBLKSIZE)
                 for (j = ch, k = DEFDACBLKSIZE, fp2 = fp1; k--;
@@ -539,6 +576,23 @@ int alsa_send_dacs(void)
                         * (1./ INT32_MAX);
             }
         }
+        else if (alsa_indev[iodev].a_sampwidth == 3)
+        {
+#if BYTE_ORDER == LITTLE_ENDIAN                                             
+            for (i = 0; i < chans; i++, ch++, fp1 += DEFDACBLKSIZE)
+            {
+                for (j = ch, k = DEFDACBLKSIZE, fp2 = fp1; k--;
+                     j += thisdevchans, fp2++)
+                    *fp2 = ((float) (
+                        (((unsigned char *)alsa_snd_buf)[3*j] << 8)
+                        | (((unsigned char *)alsa_snd_buf)[3*j+1] << 16)
+                        | (((unsigned char *)alsa_snd_buf)[3*j+2] << 24)))
+                        * (1./ INT32_MAX);
+            }
+#else
+                fprintf(stderr, "big endian 24-bit not supported");
+#endif
+        }
         else
         {
             for (i = 0; i < chans; i++, ch++, fp1 += DEFDACBLKSIZE)
