@@ -10,19 +10,6 @@ inputs to int and their outputs back to float. */
 #include <math.h>
 
 
-/* MSW and OSX don't appear to have single-precision ANSI math */
-#if defined(MSW) || defined(__APPLE__)
-#define sinf sin
-#define cosf cos
-#define atanf atan
-#define atan2f atan2
-#define sqrtf sqrt
-#define logf log
-#define expf exp
-#define fabsf fabs
-#define powf pow
-#endif
-
 typedef struct _binop
 {
     t_object x_obj;
@@ -623,7 +610,6 @@ static void log_float(t_object *x, t_float f)
     outlet_float(x->ob_outlet, r);
 }
 
-
 static t_class *exp_class;      /* ----------- exp --------------- */
 
 static void *exp_new(void)
@@ -659,6 +645,20 @@ static void abs_float(t_object *x, t_float f)
     outlet_float(x->ob_outlet, fabsf(f));
 }
 
+static t_class *wrap_class;      /* ----------- wrap --------------- */
+
+static void *wrap_new(void)
+{
+    t_object *x = (t_object *)pd_new(wrap_class);
+    outlet_new(x, &s_float);
+    return (x);
+}
+
+static void wrap_float(t_object *x, t_float f)
+{
+    outlet_float(x->ob_outlet, f - floor(f));
+}
+
 /* ------------------------  misc ------------------------ */
 
 static t_class *clip_class;
@@ -898,6 +898,11 @@ void x_arithmetic_setup(void)
     class_addfloat(abs_class, (t_method)abs_float);    
     class_sethelpsymbol(abs_class, math_sym);
 
+    wrap_class = class_new(gensym("wrap"), wrap_new, 0,
+        sizeof(t_object), 0, 0);
+    class_addfloat(wrap_class, (t_method)wrap_float);    
+    class_sethelpsymbol(wrap_class, math_sym);
+
 /* ------------------------  misc ------------------------ */
 
     clip_setup();
